import numpy as np
#from scipy import sparse
from cylp.cy import CyCoinMpsIO
from cylp.py.utils.sparseUtil import csr_matrixPlus


def readQPS(inputFilename):
    problem = CyCoinMpsIO()
    problem.readMps(inputFilename)

    n = problem.nVariables
    m = problem.nConstraints

    signs = problem.constraintSigns
    iEq = [i for i in range(len(signs)) if chr(signs[i]) == 'E']
    numberOfEqualities = len(iEq)
    iInEq = [i for i in range(len(signs)) if i not in iEq]
    numberOfInequalities = len(iInEq)

    c = problem.matrixByRow
    el = c.elements
    col = c.indices
    start = c.vectorStarts
    coefs = csr_matrixPlus((el, col, start), shape=(m, n))

    # an invalid value for initialization
    # This wont make any problems because we always
    # check number of equalties and
    # inequalities first, before accessing them
    A = C = b = c_up = c_low = 0

    rhs = problem.rightHandSide

    if numberOfEqualities:
        A = csr_matrixPlus(coefs[iEq])
        b = rhs[iEq]

    if numberOfInequalities:
        C = csr_matrixPlus(coefs[iInEq])
        c_up = problem.constraintUpper[iInEq]
        c_low = problem.constraintLower[iInEq]

    Hessian = problem.Hessian

    x_low = problem.variableLower
    x_up = problem.variableUpper

    c = problem.objCoefficients

    return (Hessian, c, A, b, C, c_low, c_up, x_low, x_up,
            n, len(iEq), len(iInEq), problem.objectiveOffset)



'''
    Module provide a facility to model a linear program.
    Although the current usage is to pass the resulting LP to CLP
    to solve it is independant of the solver.

    .. _modeling-usage:

    **Usage (single dimension, using CyLPModel indirectly)**

    >>> import numpy as np
    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPArray
    >>>
    >>> s = CyClpSimplex()
    >>>
    >>> # Add variables
    >>> x = s.addVariable('x', 3)
    >>> y = s.addVariable('y', 2)
    >>>
    >>> # Create coefficients and bounds
    >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
    >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    >>> D = np.matrix([[1., 2.],[0, 1]])
    >>> a = CyLPArray([5, 2.5])
    >>> b = CyLPArray([4.2, 3])
    >>> x_u= CyLPArray([2., 3.5])
    >>>
    >>> # Add constraints
    >>> s += A * x <= a
    >>> s += 2 <= B * x + D * y <= b
    >>> s += y >= 0
    >>> s += 1.1 <= x[1:3] <= x_u
    >>>
    >>> # Set the objective function
    >>> c = CyLPArray([1., -2., 3.])
    >>> s.objective = c * x + 2 * y.sum()
    >>>
    >>> # Solve using primal Simplex
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0.2,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0. ,  0.9])
    >>> s += x[2] + y[1] >= 2.1
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0. ,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0.,  1.])

    **Usage (single dimension, using CyLPModel directly, depending on the
      size could be faster than the indirect approach)**

    >>> import numpy as np
    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray
    >>>
    >>> model = CyLPModel()
    >>>
    >>> # Add variables
    >>> x = model.addVariable('x', 3)
    >>> y = model.addVariable('y', 2)
    >>>
    >>> # Create coefficients and bounds
    >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
    >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    >>> D = np.matrix([[1., 2.],[0, 1]])
    >>> a = CyLPArray([5, 2.5])
    >>> b = CyLPArray([4.2, 3])
    >>> x_u= CyLPArray([2., 3.5])
    >>>
    >>> # Add constraints
    >>> model += A * x <= a
    >>> model += 2 <= B * x + D * y <= b
    >>> model += y >= 0
    >>> model += 1.1 <= x[1:3] <= x_u
    >>>
    >>> # Set the objective function
    >>> c = CyLPArray([1., -2., 3.])
    >>> model.objective = c * x + 2 * y.sum()
    >>>
    >>> # Create a CyClpSimplex instance from the model
    >>> s = CyClpSimplex(model)
    >>>
    >>> # Solve using primal Simplex
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0.2,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0. ,  0.9])
    >>> s += x[2] + y[1] >= 2.1
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0. ,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0.,  1.])

    **Usage (multi-dimensions, using CyLPModel indirectly)**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPArray
    >>> s = CyClpSimplex()
    >>> x = s.addVariable('x', (5, 3, 6))
    >>> s += 2 * x[2, :, 3].sum() + 3 * x[0, 1, :].sum() >= 5
    >>> s += 0 <= x <= 1
    >>> c = CyLPArray(range(18))
    >>> s.objective = c * x[2, :, :] + c * x[0, :, :]
    >>> s.primal()
    'optimal'
    >>> sol = s.primalVariableSolution['x']
    >>> abs(sol[0, 1, 0] - 1) <= 10**-6
    True
    >>> abs(sol[2, 0, 3] - 1) <= 10**-6
    True

'''

from functools import reduce
from itertools import product
from copy import deepcopy
from operator import mul
import numpy as np
from scipy import sparse
from scipy.sparse import identity, lil_matrix
from cylp.py.utils.util import Ind, getMultiDimMatrixIndex, getTupleIndex

NUMBERS = (int, float, np.int64, np.int32, np.double)
def isNumber(n):
    return (isinstance(n, NUMBERS) or
            (isinstance(n, CyLPArray) and n.shape == ()))

# Increase max stack size from 8MB to 512MB
#resource.setrlimit(resource.RLIMIT_STACK, (2**5,-1))
#sys.setrecursionlimit(10**6)

#from cylp.py.utils.sparseUtil import sparseConcat
def I(n):
    '''
    Return a sparse identity matrix of size *n*
    '''
    if n <= 0:
        return None
    #return csc_matrixPlus(sparse.eye(n, n))
    return identity(n).tocsr()

def identitySub(var):
    '''
    Return a sparse row sub-matrix of the identity matrix
    for ``var``.
    '''
    n = var.dim
    if var.parent is not None:
        n = var.parent.dim
    if var.dim == 1:
        m = lil_matrix((1, n))
        m[0, var.indices[0]] = 1
        return m
    return I(n)[var.indices, :]

class CyLPExpr:
    __array_ufunc__ = None
    
    operators = ('>=', '<=', '==', '+', '-', '*', 'u-', 'sum')

    def __init__(self, opr='', left='', right=''):
        self.opr = opr
        self.left = left
        self.right = right
        self.expr = self
        #self.variables = []

    def __hash__(self):
        return id(self)

    def __repr__(self):
        s = '%s %s %s' % (self.left, self.right, self.opr)
        return s

    def __le__(self, other):
        v = CyLPExpr(opr="<=", right=other, left=self.expr)
        self.expr = v
        return v

    def __ge__(self, other):
        v = CyLPExpr(opr=">=", right=other, left=self.expr)
        self.expr = v
        return v

    def __eq__(self, other):
        # Check if both sides are CyLPVar, in which case a pythonic
        # comparison is meant (for dictionary keys,...)
        if (other is None):
            return False
        if isinstance(self, CyLPVar) and isinstance(other, CyLPVar):
            return id(self) == id(other)
            #return (str(self) == str(other))
        v = CyLPExpr(opr="==", right=other, left=self.expr)
        self.expr = v
        return v

    def __rmul__(self, other):
        if type(other)==np.ndarray:
            raise ValueError("You should use @ when multiplying a variable with a numpy array")
        v = CyLPExpr(opr="*", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __mul__(self, other):
        if type(other)==np.ndarray:
            raise ValueError("You should use @ when multiplying a variable with a numpy array")
        v = CyLPExpr(opr="*", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __rmatmul__(self, other):
        v = CyLPExpr(opr="*", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __matmul__(self, other):
        v = CyLPExpr(opr="*", left=self, right=other)
        v.expr = v
        self.expr = v
        return v
    
    def __rsub__(self, other):
        v = CyLPExpr(opr="-", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __sub__(self, other):
        v = CyLPExpr(opr="-", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __radd__(self, other):
        v = CyLPExpr(opr="+", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __add__(self, other):
        v = CyLPExpr(opr="+", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __neg__(self):
        v = CyLPExpr(opr="u-", right=self)
        v.expr = v
        self.expr = v
        return v

    def getPostfix(self):
        if isinstance(self, CyLPVar):
            return [self]
        left = [self.left]
        right = [self.right]
        if isinstance(self.left, CyLPExpr):
            left = self.left.getPostfix()
        if isinstance(self.right, CyLPExpr):
            right = self.right.getPostfix()

        return left + right + [self.opr]

    def evaluate(self, name=''):
        '''
        Evaluates an expression in the postfix form
        '''
        #FIXME: sometimes calling this twice is different than running it once
        tokens = self.getPostfix()
        operands = []

        cons = CyLPConstraint(name)

        for token in tokens:
            # If an operand found
            if type(token) != str or token not in CyLPExpr.operators:
                operands.append(token)
                if (isinstance(token, CyLPVar)):
                    varToBeAdded = token
                    if not varToBeAdded in cons.variables:
                        cons.variables.append(varToBeAdded)

            # Else we know what to do with an operator
            else:
                if token == 'u-':
                    right = operands.pop()
                    cons.perform(token, right=right)
                    operands.append(CyLPExpr(token, right=right))
                else:
                    right = operands.pop()
                    left = operands.pop()
                    cons.perform(token, left, right)
                    operands.append(CyLPExpr(token, left, right))

        # If something remains in the *operands* list, it means
        # that we have a single CyLPVar object on a line with no
        # operators operating on it. Check this situation and
        # add a 1 vector as its coefficient
        if len(operands) == 1 and isinstance(operands[0], CyLPVar):
            var = operands[0]
            cons.varCoefs[var] = identitySub(var) #np.ones(var.dim)

        # Reset variables for future constraints
        for var in cons.varCoefs.keys():
            var.expr = var

        return cons


class CyLPConstraint:
    gid = 0
    def __init__(self, name=''):
        self.varNames = []
        self.parentVarDims = {}
        self.varCoefs = {}
        self.lower = None
        self.upper = None
        self.nRows = None
        self.isRange = True
        self.variables = []
        if not name:
            CyLPConstraint.gid += 1
            name = 'R_%d' % CyLPConstraint.gid
        self.name = name

    def __repr__(self):
        s = '\n'
        s += 'constraint %s:\n' % self.name
        s += 'variable names:\n'
        s += str(self.varNames) + '\n'
        s += 'coefficients:\n'
        s += str(self.varCoefs) + '\n'
        s += 'lower = %s\n' % str(self.lower)
        s += 'upper = %s\n' % str(self.upper)
        if self.isRange:
            s += 'Constraint is a range\n'
        else:
            s += 'normal Constarint\n'
        return s

    def mul(self, expr, coef):
        '''
        Recursively multiplies all variable coefficients in *expr* by *coef*
        '''
        if isinstance(expr, CyLPVar):
            self.varCoefs[expr] *= coef
            return
        if isinstance(expr, CyLPExpr):
            if expr.left is None:
                return
            self.mul(expr.right, coef)
            self.mul(expr.left, coef)


    def perform(self, opr, left=None, right=None):
        if isinstance(left, (CyLPArray, np.matrix, np.ndarray)):
            left = CyLPArray(np.squeeze(np.asarray(left)))
        if isinstance(right, (CyLPArray, np.matrix, np.ndarray)):
            right = CyLPArray(np.squeeze(np.asarray(right)))

        if isinstance(right, CyLPVar):
            if right.dim == 0:
                return
            if right.name not in self.varNames:
                self.varNames.append(right.name)
                self.parentVarDims[right.name] = right.parentDim
            if opr == 'u-':
                #ones = -CyLPArray(np.ones(right.dim))
                #self.nRows = right.dim
                #self.varCoefs[right] = ones
                self.varCoefs[right] = -identitySub(right)
                self.nRows = len(right.indices)
                self.isRange = False
            elif opr == 'sum':
                n = right.dim
                if right.parent is not None:
                    n = right.parent.dim
                self.varCoefs[right] = sparse.lil_matrix((1, n))
                self.varCoefs[right][0, right.indices] = np.ones(len(right.indices))
                self.nRows = 1
                self.isRange = False
            else:
                #self.varNames.append(right.name)
                #self.parentVarDims[right.name] = right.parentDim
                dim = right.dim

                if opr == "*":  # Setting the coefficients
                    # Ignore a term if its coef is None
                    if isNumber(left):
                        if self.nRows and self.nRows != 1:
                            raise Exception("Expected 1-dimensional" \
                                            " coefficient")
                        #self.nRows = 1
                        #coef = CyLPArray(left * np.ones(dim))
                        coef = left * identitySub(right)
                        self.nRows = len(right.indices)
                    else:
                        if left is None:
                            return
                        if right.dim != left.shape[-1]:
                            raise Exception("Coefficient:\n%s\n has %d" \
                                            " columns expected %d"
                                        % (left, left.shape[1], right.dim))
                        if len(left.shape) == 1:  # Array
                            nr = 1
                        else:
                            nr = left.shape[0]
                            #nr = len(left)
                        if self.nRows and self.nRows != nr:
                            raise Exception("Coefficient:\n%s\n has %d rows" \
                                            " expected %d"
                                        % (left, left.shape[0], self.nRows))
                        self.nRows = nr
                        if right.parent is None:
                            coef = deepcopy(left)
                        else:
                            coef = sparse.lil_matrix((nr, right.parent.dim))
                            coef[:, right.indices] = left

                    self.varCoefs[right] = coef
                    self.isRange = False

                if opr == '+' or opr == '-':
                    # No coefs for left op
                    self.isRange = False
                    if isinstance(left, CyLPVar):
                        self.varCoefs[left] = identitySub(left)
                        self.nRows = len(left.indices)
                        if left.name not in self.varNames:
                            self.varNames.append(left.name)
                            self.parentVarDims[left.name] = left.parentDim

                    # No coefs for right op
                    if isinstance(right, CyLPVar):
                        if right in self.varCoefs.keys():
                            self.varCoefs[right] *= -1
                        else:
                            coef = identitySub(right)
                            if opr == '-':
                                coef *= -1
                            self.varCoefs[right] = coef
                            self.nRows = len(right.indices)
                            #if self.nRows == 0:
                            #    self.nRows = 1
                            #    coef = CyLPArray(np.ones(right.dim))
                            #elif self.nRows == 1:
                            #    coef = CyLPArray(np.ones(1))
                            #else:
                            #    coef = np.matrix(np.eye(self.nRows))

                            #if opr == '-':
                            #    coef *= -1
                            #self.varCoefs[right] = coef
                            if right.name not in self.varNames:
                                self.varNames.append(right.name)
                                self.parentVarDims[right.name] = \
                                                            right.parentDim
        # check if no coef for left
        if left.__class__ == CyLPVar and (opr == '-' or opr == '+'):
            if left.dim == 0 :
                return
            self.isRange = False
            #ones = CyLPArray(np.ones(left.dim))
            #self.varCoefs[left] = ones
            #self.nRows = 1
            self.varCoefs[left] = identitySub(left)
            self.nRows = len(left.indices)
            if left.name not in self.varNames:
                self.varNames.append(left.name)
                self.parentVarDims[left.name] = left.parentDim

        # Coef already set for the right operand
        if right.__class__ == CyLPExpr:
            if opr == '-':
                # The expression on the right is in the form (left opr right)
                # so the key to the coef in self.varCoefs is right.right i.e.
                # the CyLPVar on the right
                #if right.right.dim == 0:
                #    return
                self.isRange = False
                self.mul(right, -1)
                #self.varCoefs[right.right] *= -1
            if opr == '*' and isNumber(left):
                self.mul(right, left)


        if opr in ('<=', '>=', '=='):
            if isinstance(left, CyLPExpr) and not isinstance(right,
                                                              CyLPExpr):
                bound = right
            elif isinstance(right, CyLPExpr) and not isinstance(left,
                                                                CyLPExpr):
                bound = left
            else:
                raise Exception('At least one side of a comparison sign' \
                                'should be constant.')

            # FIXME: check this: I suppose never runs
            if self.isRange:
                var = self.variables[0]
                self.nRows = var.dim
                dim = var.dim
            #if not self.nRows:
                #self.nRows = left.parentDim
                #dim = left.parentDim
            else:
#                # FIXME: Think of the correct way to check bound dimensions
#                if (not isinstance(right, (float, int)) and
#                            right.shape[0] != self.nRows):
#                    #raise Exception('Bound dim: %d, expected dim: %d '
#                    #                % (right.shape[0], self.nRows))
#                    pass
                dim = self.nRows
#
            if isNumber(right):
                bound = CyLPArray(right * np.ones(dim))

            if self.isRange:
                if ((opr in ('>=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('<=', '==') and isinstance(right, CyLPExpr))):
                    if var.parent:
                        var.parent.lower[var.indices] = bound
                    else:
                        var.lower[var.indices] = bound

                if ((opr in ('<=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('>=', '==') and isinstance(right, CyLPExpr))):
                    if var.parent:
                        var.parent.upper[var.indices] = bound
                    else:
                        var.upper[var.indices] = bound

            else:
                if ((opr in ('>=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('<=', '==') and isinstance(right, CyLPExpr))):
                    self.lower = bound
                    if self.upper is None:
                        self.upper = getCoinInfinity() * np.ones(len(bound))
                if ((opr in ('<=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('>=', '==') and isinstance(right, CyLPExpr))):
                    self.upper = bound
                    if self.lower is None:
                        self.lower = -getCoinInfinity() * np.ones(len(bound))


class CyLPVar(CyLPExpr):
    '''
    Contains variable information such as its name, dimension,
    bounds, and whether or not it is an integer. We don't creat
    instances directly but rather use :py:func:`CyLPModel.addVariable`
    to create one.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __init__(self, name, dim, isInt=False, fromInd=None, toInd=None):
        self.name = name
        self.dims = None
        if isinstance(dim, tuple):
            self.setDims(dim)
            self.dim = dim = reduce(mul, dim)
            self.parentDim = dim
        else:
            self.dim = dim
            self.parentDim = dim
        self.parent = None
        self.isInt = isInt
        self.expr = self
        self.lower = -getCoinInfinity() * np.ones(dim)
        self.upper = getCoinInfinity() * np.ones(dim)

        self.indices = np.arange(dim, dtype=np.int32)
        if fromInd and toInd:
            #self.indices = np.arange(fromInd, toInd, dtype=np.int32)
            self.fromInd = fromInd
            self.toInd = toInd
        else:
            self.fromInd = 0
            self.toInd = dim-1

    def __repr__(self):
        s = self.name
        if self.fromInd and self.toInd:
            s += '[%d:%d]' % (self.fromInd, self.toInd)
        elif self.parent is not None and len(self.indices) == 1:
            s += '[%d]' % self.indices
        return s

    def setDims(self, ds):
        self.dims = ds

    def __getitem__(self, key):
        if type(key) == int:
            newObj = CyLPVar(self.name, 1)
            newObj.indices = np.array([key], np.int32)
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.dim = 1
        elif type(key) == slice:
            newObj_fromInd = key.start
            if key.stop > self.dim:
                newObj_toInd = self.dim
            else:
                newObj_toInd = key.stop

            newObj = CyLPVar(self.name, newObj_toInd - newObj_fromInd)
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.fromInd = newObj_fromInd
            newObj.toInd = newObj_toInd
            newObj.indices = np.arange(newObj.fromInd, newObj.toInd,
                                       dtype='int32')
            # Save parentDim for future use
            newObj.dim = len(newObj.indices)
        elif isinstance(key, (np.ndarray, list)):
            newObj = CyLPVar(self.name, len(key))
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.fromInd = None
            newObj.toInd = None
            newObj.indices = np.array(key)
            newObj.dim = len(newObj.indices)

        elif isinstance(key, tuple):
            inds = []
            n = range(len(key))
            for i in n:
                k = key[i]
                if isinstance(k, int):
                    inds.append(Ind(slice(k, k+1), self.dims[i]))
                else:
                    inds.append(Ind(k, self.dims[i]))
            newObj = self.__getitem__(getMultiDimMatrixIndex(inds))
        return newObj

    def sum(self):
        v = CyLPExpr(opr="sum", right=self)
        v.expr = v
        self.expr = v
        return v


class CyLPArray(np.ndarray):
    '''
    It is a tweaked Numpy array. It allows user to define objects
    comparability to an array. If we have an instance of ``CyLPVar``
    called ``x`` and a numpy array ``b``, then ``b >= x`` returns an array
    of the same dimension as ``b`` with all elements equal to ``False``,
    which has no sense at all. On the constrary ``CyLPArray`` will return
    ``NotImplemented`` if it doesn't know how to compare. This is particularly
    helpful when we define LP constraints.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __new__(cls, input_array, info=None):
        # Input array is an already formed ndarray instance
        # We first cast to be our class type
        obj = np.asarray(input_array).view(cls)
        # add the new attribute to the created instance
        obj.__le__ = CyLPArray.__le__
        obj.__ge__ = CyLPArray.__ge__
        return obj

    def __le__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__le__(self, other)

    def __ge__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__ge__(self, other)

    def __mul__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__mul__(self, other)

    def __rmul__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__rmul__(self, other)

    def __add__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__add__(self, other)

    def __radd__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__radd__(self, other)

    def __rsub__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__rsub__(self, other)

    def __sub__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__sub__(self, other)


class IndexFactory:
    '''
    A small class that keeps track of the indices of variables and constraints
    that you are adding to a problem gradually.

    **Usage**

    >>> import numpy as np
    >>> from cylp.py.modeling.CyLPModel import IndexFactory
    >>> i = IndexFactory()
    >>> i.addVar('x', 10)
    >>> i.addVar('y', 5)
    >>> i.hasVar('x')
    True
    >>> (i.varIndex['y'] == np.arange(10, 15)).all()
    True
    >>> i.addConst('Manpower', 4)
    >>> i.addConst('Gold', 10)
    >>> (i.constIndex['Manpower'] == np.arange(4)).all()
    True
    >>> (i.constIndex['Gold'] == np.arange(4, 14)).all()
    True

    '''

    def __init__(self):
        self.varIndex = {}
        self.constIndex = {}
        self.currentVarIndex = 0
        self.currentConstIndex = 0

    def addVar(self, varName, numberOfVars):
        if numberOfVars == 0:
            return
        if not varName:
            raise Exception('You must specify a name for a variable.')
        if varName in self.varIndex.keys():
            print('Variable already exists.')
            #self.varIndex[varName] += range(self.currentVarIndex,
            #                                self.currentVarIndex +
            #                                numberOfVars)
        else:
            self.varIndex[varName] = np.arange(self.currentVarIndex,
                                            self.currentVarIndex +
                                            numberOfVars, dtype='int32')
        self.currentVarIndex += numberOfVars

    def removeVar(self, name):
        if not self.hasVar(name):
            raise Exception('Variable "%s" does not exist.' % name)
        nVars = len(self.varIndex[name])
        start = self.varIndex[name][0]
        del self.varIndex[name]
        for varName in self.varIndex.keys():
            inds = self.varIndex[varName]
            if inds[0] > start:
                self.varIndex[varName] = inds - nVars * np.ones(len(inds),
                                                                np.int32)

    def hasVar(self, varName):
        return varName in self.varIndex.keys()

    def hasConst(self, constName):
        return constName in self.constIndex

    def getLastVarIndex(self):
        return self.currentVarIndex - 1

    def addConst(self, constName, numberOfConsts):
        if numberOfConsts == 0:
            return
        if not constName:
            raise Exception('You must specify a name for a constraint.')
        if self.hasConst(constName):
            print('Constraint already exists: %s' % constName)
            #self.constIndex[constName] += range(self.currentConstIndex,
            #        self.currentConstIndex + numberOfConsts)
        else:
            self.constIndex[constName] = np.arange(self.currentConstIndex,
                                            self.currentConstIndex +
                                            numberOfConsts, dtype='int32')
        self.currentConstIndex += numberOfConsts

    def removeConst(self, name):
        if not self.hasConst(name):
            raise Exception('Constraint "%s" does not exist.' % name)
        nCons = len(self.constIndex[name])
        start = self.constIndex[name][0]
        del self.constIndex[name]
        for constName in self.constIndex.keys():
            inds = self.constIndex[constName]
            if inds[0] > start:
                self.constIndex[constName] = inds - nCons * np.ones(len(inds),
                                                                    np.int32)
        self.currentConstIndex -= nCons


    def getLastConstIndex(self):
        return self.currentConstIndex - 1

    def __repr__(self):
        varind = self.varIndex
        cind = self.constIndex

        s = "variables : \n"
        for vname, rg in varind.items():
            s += '%s : %s\n' % (vname.rjust(15), str(rg))
        s += '\n'
        s += "constraints : \n"
        for cname, rg in cind.items():
            s += '%s : %s\n' % (cname.rjust(15), str(rg))
        return s

    def reverseVarSearch(self, ind):
        '''
        Take an index and return the corresponding variable name.
        '''
        inds = self.varIndex
        for key in inds.keys():
            if ind in inds[key]:
                i = np.where(inds[key]==ind)[0][0]
                return key, i
        return -1, -1


from cylp.py.utils.sparseUtil import sparseConcat, csr_matrixPlus, csc_matrixPlus

class CyLPModel(object):
    '''
    Hold all the necessary information to create a linear program, i.e.
    variables, constraints, and the objective function.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __init__(self):
        self.variables = []
        self.constraints = []
        self.objective_ = None
        self.inds = IndexFactory()
        self.nVars = 0
        self.varNames = []
        self.nCons = 0
        self.pvdims = {}

    def addVariable(self, name, dim, isInt=False):
        '''
        Create a new instance of :py:class:`CyLPVar` using the given
        arguments and add it to current model's variable list.
        '''

        if dim == 0:
            return

        #If mulidim, correct dim
        if isinstance(dim, tuple):
            numVars = reduce(mul, dim)
        else:
            numVars = dim
        var = CyLPVar(name, dim, isInt, self.nVars, self.nVars+numVars)
        self.variables.append(var)

        if not self.inds.hasVar(var.name):
            self.inds.addVar(var.name, numVars)
            self.nVars += numVars
            self.varNames.append(var.name)
            self.pvdims[var.name] = numVars

            if var.dims:
                var.mpsNames = [var.name + '_' + '_'.join(x) for x in \
                        product(*[map(str, range(i)) for i in var.dims])]
            else:
                var.mpsNames = ['%s_%s' % (var.name, i) for i in range(var.dim)]

            o = self.objective_
            if isinstance(o, np.ndarray):
                o = np.concatenate((o, np.zeros(numVars)), axis=0)
            else:
                o = sparseConcat(o, csr_matrixPlus(np.zeros(numVars)), 'h')

            # I'm not exactly sure why the objective gets changed into
            # csr_matrixPlus here when it may not be coming in. Shouldn't it
            # just always be?
            self.objective_ = csr_matrixPlus(o)

        else:
            raise Exception('Varaible %s already exists.' % var.name)

        return var

    def removeVariable(self, name):
        '''
        Remove a variable named ``name`` from the model
        '''
        if not self.inds.hasVar(name):
            raise Exception('Variable "%s" does not exist.' % name)

        self.nVars -= self.pvdims[name]
        start = self.inds.varIndex[name][0]
        end = start + self.pvdims[name]
        o = self.objective_

        if isinstance(o, np.ndarray):
            o = np.concatenate((o[:start], o[end:]), axis=0)
        else:
            if end == o.shape[1]:
                if start == 0:
                    print('Problem empty.')
                else:
                    o = o[0, :start]
            elif start == 0:
                o = o[0, end:]
            else:
                o = sparseConcat(o[0, :start], o[0, end:], how='h')

        # I'm not exactly sure why the objective gets changed into
        # csr_matrixPlus here when it may not be coming in. Shouldn't it
        # just always be?
        self.objective_ = csr_matrixPlus(o)

        del self.pvdims[name]
        self.varNames.remove(name)
        self.inds.removeVar(name)
        for i in range(len(self.variables)):
            var = self.variables[i]
            if var.name == name:
                del self.variables[i]
                break

        #Removing the variable from the constraints
        cons = self.constraints
        for c in cons:
            if name in c.varNames:
                c.varNames.remove(name)
                del c.parentVarDims[name]
                for v in list(c.varCoefs.keys()):
                    if v.name == name:
                        del c.varCoefs[v]

        for c in cons[:]:
            self.inds
            if not c.varCoefs:
                self.removeConstraint(c.name)

    def getVarByName(self, varName):
        '''
        Return a variable with name ``varName``.
        '''
        for var in self.variables:
            if var.name == varName:
                return var
        return None

    def makeIndexFactory(self):
        self.allVarNames = []
        varNameDict = {} #copy of allVarNames in dictionary form, for efficiency
        self.allParentVarDims = {}
        self.nRows = 0
        for c in self.constraints:
            # Make sure to add variables in the same order they were
            # added to CyLPModel
            for var in self.variables:
                for vname in c.varNames:
                    if vname == var.name and vname not in varNameDict:
                        self.allVarNames += [vname]
                        varNameDict[vname] = True
            self.allParentVarDims.update(c.parentVarDims)
            self.nRows += c.nRows

    @property
    def objective(self):
        return self.objective_

    @objective.setter
    def objective(self, obj):
        if isinstance(obj, CyLPExpr):
            self.objective_ = obj.evaluate()
            obj = None
            #for varName in self.allVarNames:
            for varName in self.varNames:
                v_coef = self.generateVarObjCoef(varName)
                obj = sparseConcat(obj, v_coef, how='h')
            obj = np.squeeze(np.asarray(obj.todense()))
        self.objective_ = obj

    def evaluateAt(self, x0):
        return np.dot(self.objective, x0)

    def __iadd__(self, cons):
        '''
        Call :meth:`addConstraint`. The only difference is that
        you cannot specify a name for your constraint.
        '''
        self.addConstraint(cons)
        return self

    def addConstraint(self, cons, consName='', addMpsNames=True):
        '''
        Add constraint ``cons`` to the ``CyLPModel``. Argument ``cons`` must
        be an expresion made using instances of :py:class:`CyLPVar`,
        :py:class:`CyLPArray`, Numpy matrices, or sparse matrices. It can
        use normal operators such as ``>=``, ``<=``, ``==``, ``+``, ``-``,
        ``*``.

        See the :ref:`modeling example <modeling-usage>`.

        '''

        c = cons.evaluate(consName)
        if not c.isRange:
            self.constraints.append(c)
            if c.name:
                self.inds.addConst(c.name, c.nRows)
            self.nCons += c.nRows

        if addMpsNames:
            c.mpsNames = []
            for i in range(c.nRows):
                c.mpsNames.append('%s_%s' % (c.name, str(i)))


        #self.makeMatrices()
        return c

    def removeConstraint(self, name):
        '''
        Remove constraint called ``name`` and return its CLP indices back
        for actual removal.
        '''
        if not self.inds.hasConst(name):
            raise Exception('Constraint "%s" does not exist.' % name)

        for i in range(len(self.constraints)):
            if self.constraints[i].name == name:
                con = self.constraints[i]
                break
        self.nCons -= con.nRows
        del self.constraints[i]
        indsOfConstriantsToBeRemoved = self.inds.constIndex[name]
        self.inds.removeConst(name)
        return indsOfConstriantsToBeRemoved

    def generateVarObjCoef(self, varName):
        #dim = self.allParentVarDims[varName]
        dim = self.pvdims[varName]
        coef = csr_matrixPlus((1, dim))
        obj = self.objective_
        keys = [k for k in obj.varCoefs.keys() if k.name == varName]

        for var in keys:
            coef = coef + obj.varCoefs[var]
        return coef

    def generateVarMatrix(self, varName):
        '''
        Create the coefficient matrix for a variable named
        varName. Return a csr_matrixPlus.
        '''
        #dim = self.allParentVarDims[varName]
        dim = self.pvdims[varName]
        mainCoef = None
        for c in self.constraints:
            coef = sparse.coo_matrix((c.nRows, dim))
            keys = [k for k in c.varCoefs.keys() if k.name == varName]
            for var in keys:
                coef = coef + c.varCoefs[var]
            mainCoef = sparseConcat(mainCoef, coef, 'v')

        return mainCoef

    def makeMatrices(self):
        '''
        Makes coef matrix and rhs vector from CyLPConstraints
        in self.constraints
        '''
        #if len(self.constraints) == 0:
        #    return
        self.makeIndexFactory()
        #self.getVarBounds()


        # Create the aggregated coef matrix
        masterCoefMat = None

        masterCoefMat = None
        if self.nCons > 0:
            for varName in self.varNames:# self.pvdims.keys():#self.allVarNames:
                vmat = self.generateVarMatrix(varName)

                if vmat is None:
                    vmat = csc_matrixPlus((self.nCons, self.pvdims[varName]))
                masterCoefMat = sparseConcat(masterCoefMat, vmat, 'h')

        # Create bound vectors
        c_lower = np.array([])
        c_upper = np.array([])
        for c in self.constraints:
            if not c.isRange:
                c_lower = np.concatenate((c_lower, c.lower), axis=0)
                c_upper = np.concatenate((c_upper, c.upper), axis=0)
        # Create variables bound vectors
        v_lower = np.array([])
        v_upper = np.array([])
        if self.nVars:
            v_lower = -getCoinInfinity() * np.ones(self.nVars)
            v_upper = getCoinInfinity() * np.ones(self.nVars)
            for v in self.variables:
                varinds = self.inds.varIndex[v.name]
                v_lower[varinds] = v.lower
                v_upper[varinds] = v.upper

                #v_lower = np.concatenate((v_lower, v.lower), axis=0)
                #v_upper = np.concatenate((v_upper, v.upper), axis=0)
        #if masterCoefMat is not None:
        return masterCoefMat, c_lower, c_upper, v_lower, v_upper


class CyLPSolution:

    def __init__(self):
        self.sol = {}

    def add(self, key, val):
        'Add (key, val) to self.dic. Ignore if val is zero'
        if val == 0: # See if necessary to use a tolerance
            return
        self.sol[key] = val

    def getVal(self, key):
        'Return the value corresponing key'
        if key not in self.sol.keys():
            return 0
        return self.sol[key]

    def __getitem__(self, key):
        #if isinstance(key, tuple):
        if key in self.sol.keys():
            return self.sol[key]
        else:
            return 0

    def __setitem__(self, key, val):
        #if isinstance(key, tuple):
        if val == 0: # See if necessary to use a tolerance
            return
        self.sol[key] = val
    def __repr__(self):
        return repr(self.sol)

def getCoinInfinity():
    return 1.79769313486e+308


if __name__ == '__main__':
    from cylp.cy import CyClpSimplex
    from cylp.py.modeling.CyLPModel import CyLPArray
    s = CyClpSimplex()
    x = s.addVariable('x', (5, 3, 6))
    s += 2 * x[2, :, 3].sum() + 3 * x[0, 1, :].sum() >= 5

    s += 0 <= x <= 1
    c = CyLPArray(range(18))

    s.objective = c * x[2, :, :] + c * x[0, :, :]
    s.writeMps('/Users/mehdi/Desktop/test.mps')
    s.primal()
    sol = s.primalVariableSolution
    print(sol)

#model = CyLPModel()
#
#x = model.addVariable('x', 5)
#y = model.addVariable('y', 4)
#z = model.addVariable('z', 5)
#
#
#b = CyLPArray([3.1, 4.2])
#aa = np.matrix([[1, 2, 3, 3, 4], [3, 2, 1, 2, 5]])
#aa = np.matrix([[0, 0, 0, -1.5, -1.5], [-3, 0, 0, -2,0 ]])
#dd = np.matrix([[1, 3, 1, 4], [2, 1,3, 5]])
#
#
#model.addConstraint(1.5 <=  x[1:3] + 5 * y[3:] - 6 * x[2:5]  + x[0] <= 14)
#
#print(model.constraints[0].varCoefs)
#
#model.addConstraint(2 <= aa*x + dd * y <= b)
#model.addConstraint(x[2:4] >= 3)
#model.addConstraint(1.1 <= y <= CyLPArray([1,2,3,4]))
#
#
#A = model.makeMatrices()
#
#print('x:')
#print(x.lower)
#print(x.upper)
#print('y:')
#print(y.lower)
#print(y.upper)
#
#print('+++++++++++++++++++++++++++++')
##cc = x[0] - 2*x[1:3] -2.5 * y[1]
##cc = cc.evaluate()
#model.objective = x[0] - 2*x[1:3] -2.5 * y[1]
##print(model.allParentVarDims)
##print(model.generateVarObjCoef('x'))
##print(model.generateVarObjCoef('y'))
#
#print(model.objective)
##print(cl)
##print(cu)



import numpy as np
cimport numpy as np
from cylp.cy.CyClpDualRowPivotBase cimport *
cimport cython.operator.dereference as deref
from cylp.cy cimport CyClpSimplex
from cylp.cy.CyCoinIndexedVector cimport CyCoinIndexedVector
from cpython cimport Py_INCREF, Py_DECREF

cdef class CyDualPivotPythonBase(CyClpDualRowPivotBase):
    cdef pivotRow(self)
    cdef CyClpDualRowPivot * clone(self, bint copyData)
    cdef double updateWeights(self, CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn)

    cdef void updatePrimalSolution(self, CppCoinIndexedVector* inp,
                                         double theta,
                                         np.ndarray[np.double_t,ndim=1] changeInObjective)

    cdef object dualPivotMethodObject



import numpy as np
from operator import itemgetter
from random import shuffle
from math import floor
from .PivotPythonBase import PivotPythonBase
from cylp.cy.CyClpSimplex import VarStatus


class LIFOPivot(PivotPythonBase):
    '''
    Last-In-First-Out pivot rule implementation.

    **Usage**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.pivots import LIFOPivot
    >>> from cylp.py.pivots.LIFOPivot import getMpsExample
    >>> # Get the path to a sample mps file
    >>> f = getMpsExample()
    >>> s = CyClpSimplex()
    >>> s.readMps(f)  # Returns 0 if OK
    0
    >>> pivot = LIFOPivot(s)
    >>> s.setPivotMethod(pivot)
    >>> s.primal()
    'optimal'
    >>> round(s.objectiveValue, 5)
    2520.57174

    '''

    def __init__(self, clpModel):
        self.dim = clpModel.nRows + clpModel.nCols
        self.clpModel = clpModel
        #self.banList = np.zeros(self.dim, np.int)
        self.banList = []
        self.priorityList = range(self.dim)

    def pivotColumn(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        'Finds the variable with the best reduced cost and returns its index'
        self.updateReducedCosts(updates, spareRow1, spareRow2, spareCol1, spareCol2)
        s = self.clpModel
        rc = s.reducedCosts
        dim = s.nCols + s.nRows

        tol = s.dualTolerance

        for i in self.priorityList:
            #flagged or fixed
            if s.flagged(i) or s.CLP_getVarStatus(i) == VarStatus.fixed:
                continue

            #TODO: can we just say dualInfeasibility = rc[i] ** 2
            if s.CLP_getVarStatus(i) == VarStatus.atUpperBound:  # upperbound
                dualInfeasibility = rc[i]
            elif (s.CLP_getVarStatus(i) == VarStatus.superBasic or
                    s.CLP_getVarStatus(i) == VarStatus.free):  # free or superbasic
                dualInfeasibility = abs(rc[i])
            else:  # lowerbound
                dualInfeasibility = -rc[i]

            if dualInfeasibility > tol:
                return i

        return -1

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
        '''
        Inserts the leaving variable index as the first element
        in self.priorityList
        '''
        s = self.clpModel

        pivotRow = s.pivotRow()
        if pivotRow < 0:
            return True

        pivotVariable = s.getPivotVariable()
        leavingVarIndex = pivotVariable[pivotRow]

        self.priorityList.remove(leavingVarIndex)
        self.priorityList.insert(0, leavingVarIndex)

        return True


def getMpsExample():
    import os
    import inspect
    cylpDir = os.environ['CYLP_SOURCE_DIR']
    return os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')



# cython: embedsignature=True


cdef class CyPivotPythonBase(CyClpPrimalColumnPivotBase):
    def __init__(self, pivotMethodObject):
        CyClpPrimalColumnPivotBase.__init__(self)
        Py_INCREF(pivotMethodObject)
        self.pivotMethodObject = pivotMethodObject

    def __dealloc__(self):
        Py_DECREF(self.pivotMethodObject)

    cdef pivotColumn(self, CppCoinIndexedVector* updates,
                     CppCoinIndexedVector* spareRow1, CppCoinIndexedVector* spareRow2,
                     CppCoinIndexedVector* spareCol1, CppCoinIndexedVector* spareCol2):
        cyupdates = CyCoinIndexedVector()
        cyupdates.setCppSelf(updates)
        cyspareRow1 = CyCoinIndexedVector()
        cyspareRow1.setCppSelf(spareRow1)
        cyspareRow2 = CyCoinIndexedVector()
        cyspareRow2.setCppSelf(spareRow2)
        cyspareCol1 = CyCoinIndexedVector()
        cyspareCol1.setCppSelf(spareCol1)
        cyspareCol2 = CyCoinIndexedVector()
        cyspareCol2.setCppSelf(spareCol2)
        return self.pivotMethodObject.pivotColumn(cyupdates,
                                    cyspareRow1, cyspareRow2,
                                    cyspareCol1, cyspareCol2)

    cdef CyClpPrimalColumnPivot* clone(self, bint copyData):
        cdef CyClpPrimalColumnPivot* ret =  \
                <CyClpPrimalColumnPivot*> new CppClpPrimalColumnPivotBase(
                            <cpy_ref.PyObject*>self,
                            <runPivotColumn_t>RunPivotColumn,
                            <runClone_t>RunClone,
                            <runSaveWeights_t>RunSaveWeights)
        return ret

    cdef void saveWeights(self, CppIClpSimplex * model, int mode):
        cymodel = CyClpSimplex()
        cymodel.setCppSelf(model)
        self.pivotMethodObject.saveWeights(cymodel, mode)




#from cylp.cy cimport CyCgl


cdef class CyCglCutGenerator:
    pass

cdef class CyCglAllDifferent:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglAllDifferent()

cdef class CyCglClique:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglClique()

cdef class CyCglKnapsackCover:
    def __cinit__(self, maxInKnapsack=50):
        self.CppSelf = <CppCglCutGenerator*>new CppCglKnapsackCover()
        self.maxInKnapsack = maxInKnapsack

    cdef CppCglKnapsackCover* realCppSelf(self):
        return <CppCglKnapsackCover*>self.CppSelf

    property maxInKnapsack:
        def __get__(self):
            return self.realCppSelf().getMaxInKnapsack()

        def __set__(self, limit):
            self.realCppSelf().setMaxInKnapsack(limit)

cdef class CyCglOddHole:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglOddHole()

#######################

cdef class CyCglFlowCover:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglFlowCover()

######################

cdef class CyCglGomory(CyCglCutGenerator):
    def __cinit__(self, limit=50):
        self.CppSelf = <CppCglCutGenerator*>new CppCglGomory()
        self.limit = limit

    cdef CppCglGomory* realCppSelf(self):
        return <CppCglGomory*>self.CppSelf

    property limit:
        def __get__(self):
            return self.realCppSelf().getLimit()

        def __set__(self, limit):
            self.realCppSelf().setLimit(limit)

cdef class CyCglRedSplit:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglRedSplit()

##############################

cdef class CyCglLiftAndProject:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglLiftAndProject()

cdef class CyCglLandP:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglLandP()

#################################

cdef class CyCglMixedIntegerRounding:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglMixedIntegerRounding()

cdef class CyCglMixedIntegerRounding2:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglMixedIntegerRounding2()

cdef class CyCglTwomir:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglTwomir()


cdef class CyCglResidualCapacity:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglResidualCapacity()


###########################


#cdef class CyCglDuplicateRow:
#    def __cinit__(self):
#        self.CppSelf = <CppCglCutGenerator*>new CppCglDuplicateRow()

cdef class CyCglPreProcess:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglPreProcess()

cdef class CyCglProbing:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglProbing()

cdef class CyCglSimpleRounding:
    def __cinit__(self):
        self.CppSelf = <CppCglCutGenerator*>new CppCglSimpleRounding()





# cython: embedsignature=True


cdef class CyWolfePivot(CyClpPrimalColumnPivotBase):

    cdef pivotColumn(self, CppCoinIndexedVector* cppupdates, CppCoinIndexedVector* cppspareRow1,
                    CppCoinIndexedVector* cppspareRow2, CppCoinIndexedVector* cppspareCol1,
                    CppCoinIndexedVector* cppspareCol2):
        updates = CyCoinIndexedVector()
        updates.setCppSelf(cppupdates)
        spareRow1 = CyCoinIndexedVector()
        spareRow1.setCppSelf(cppspareRow1)
        spareRow2 = CyCoinIndexedVector()
        spareRow2.setCppSelf(cppspareRow2)
        spareCol1 = CyCoinIndexedVector()
        spareCol1.setCppSelf(cppspareCol1)
        spareCol2 = CyCoinIndexedVector()
        spareCol2.setCppSelf(cppspareCol2)

        m = self.cyModel

        # Update the reduced costs, for both the original and the slack variables

        s = self.cyModel

        if updates.nElements:
            m.updateColumnTranspose(spareRow2, updates)
            m.transposeTimes(-1, updates, spareCol2, spareCol1)
            m.reducedCosts[s.nVariables:][updates.indices] -= updates.elements[:updates.nElements]
            m.reducedCosts[:s.nVariables][spareCol1.indices] -= spareCol1.elements[:spareCol1.nElements]
        updates.clear()
        spareCol1.clear()

        # cdef CyClpSimplex.Status s
        cdef double* reducedCosts = self.getReducedCosts()
        cdef int dim = self.nCols() + self.nRows()

        cdef double bestDj = self.cyModel.dualTolerance
        cdef int bestSequence = -1
        cdef i = -1
        while i < dim - 1:
            i += 1
            if self.model().flagged(i):
                continue
            rc = reducedCosts[i]
            s = self.CppSelf.model().getStatus(i)
            if s == 1 or s == 5:
                continue
            elif s == 2:
                if rc > bestDj:
                    bestDj = rc
                    bestSequence = i
                    return bestSequence
            elif s == 3:
                if rc < -bestDj:
                    bestDj = -rc
                    bestSequence = i
                    return bestSequence
        return bestSequence

    cdef CyClpPrimalColumnPivot* clone(self, bint copyData):
        cdef CyClpPrimalColumnPivot* ret =  \
                <CyClpPrimalColumnPivot*> new CppClpPrimalColumnPivotBase(
                            <cpy_ref.PyObject*>self,
                            <runPivotColumn_t>RunPivotColumn,
                            <runClone_t>RunClone,
                            <runSaveWeights_t>RunSaveWeights)
        return ret

    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex * model, int mode):
        self.CppSelf.setModel(model)



# cython: embedsignature=True

cdef void RunGenerateCuts(void *ptr, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info):
    (<CyCglCutGeneratorBase>(ptr)).generateCuts(si, cs, info)

cdef CppCglCutGenerator* RunCglClone(void *ptr):
    return (<CyCglCutGeneratorBase>(ptr)).clone()


cdef class CyCglCutGeneratorBase:
    def __init__(self):
        Py_INCREF(self)
        self.CppSelf = new CppCglCutGeneratorBase(
            <cpy_ref.PyObject*>self,
            <runGenerateCuts_t>RunGenerateCuts,
            <runCglClone_t>RunCglClone)

    def __dealloc__(self):
        Py_DECREF(self)
        del self.CppSelf

    cdef generateCuts(self, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info):
        raise Exception('CyCglCutGenerator.pyx: generateCuts must' \
                        ' be implemented.')

    cdef CppCglCutGenerator* clone(self):
        cdef CppCglCutGenerator* ret =  \
                <CppCglCutGenerator*> new CppCglCutGeneratorBase(
                                            <cpy_ref.PyObject*>self,
                                            <runGenerateCuts_t>RunGenerateCuts,
                                            <runCglClone_t>RunCglClone)
        return ret

#    cdef CyClpSimplex.CppIClpSimplex* model(self):
#        return self.CppSelf.model()
#
#    cdef void setModel(self, CyClpSimplex.CppIClpSimplex* m):
#        self.CppSelf.setModel(m)
#
#    property nRows:
#        def __get__(self):
#            return self.CppSelf.model().getNumRows()
#
#    property nCols:
#        def __get__(self):
#            return self.CppSelf.model().getNumCols()
#



import random
import numpy as np
from cylp.cy import CyCoinIndexedVector
from cylp.py.pivots.PivotPythonBase import PivotPythonBase


class WolfePivotPE(PivotPythonBase):

    def __init__(self, clpModel, bucketSize=1):
        self.dim = clpModel.nConstraints + clpModel.nVariables
        self.clpModel = clpModel
        # Require extra check after leaving variable is chosen
        clpModel.useCustomPrimal(True)
        #self.banList = np.array([])
        self.orgBan = np.array(self.dim * [True], np.bool)
        self.notBanned = self.orgBan.copy()
        self.complementarityList = np.arange(self.dim)


        #Positive-edge-related attributes
        self.isDegenerate = False

        # Create some numpy arrays here ONCE to prevent memory
        # allocation at each iteration
        self.aColumn = CyCoinIndexedVector()
        self.aColumn.reserve(self.dim)
        self.w = CyCoinIndexedVector()
        self.w.reserve(self.clpModel.nRows)

        self.rhs = np.empty(self.clpModel.nRows, dtype=np.double)
        self.EPSILON = 10**-7
        self.lastUpdateIteration = 0

        self.compCount = 0
        self.nonCompCount = 0
        self.compRej = 0
        self.numberOfIncompSinceLastUpdate = 0
        self.last_p_count = 0
        self.iCounter = 0
        self.iInterval = 100
    # Begining of Positive-Edge-related attributes

    def updateP(self):
        '''Finds constraints with abs(rhs) <=  epsilon and put
        their indices in "z"
        '''
        s = self.clpModel
        nRows = s.nRows

        rhs = self.rhs
        s.getRightHandSide(rhs)

        #self.p = np.where(np.abs(rhs) > self.EPSILON)[0]
        self.z = np.where(np.abs(rhs) <= self.EPSILON)[0]
        print('degeneracy level : ', (len(self.z)) / float(nRows))
        #self.isDegenerate = (len(self.p) != nRows)
        self.isDegenerate = (len(self.z) > 0)

    def updateW(self):
        '''Sets "w" to be a vector of random vars with "0"
        at indices defined in "p"
        Note that vectorTimesB_1 changes "w"
        '''
        self.updateP()
        self.w.clear()
        self.w[self.z] = np.random.random(len(self.z))
        s = self.clpModel
        s.vectorTimesB_1(self.w)

        self.lastUpdateIteration = s.iteration

    def random(self):
        'Defines how random vector "w" components are generated'
        return random.random()

    def isCompatible(self, varInd):
        if not self.isDegenerate:
            return False
        s = self.clpModel
        s.getACol(varInd, self.aColumn)

        return abs(cylp.cy.CyClpSImplex.cydot(self.aColumn, self.w)) < self.EPSILON

    # End of Positive-Edge-related attributes

    def pivotColumn(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        self.updateReducedCosts(updates, spareRow1, spareRow2, spareCol1, spareCol2)
        s = self.clpModel

        self.CompIter = True

        rc = s.reducedCosts
        #tol = s.dualTolerance
        tol = 0
        indicesToConsider = np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree) &
                                     self.notBanned)[0]

        rc2 = abs(rc[indicesToConsider])

        checkFree = False

        maxRc = maxCompRc = maxInd = maxCompInd = -1


        if self.isDegenerate:
            w = self.w.elements
            compatibility = np.zeros(s.nCols + s.nRows, dtype=np.double)
            if len(indicesToConsider) > 0:
                s.transposeTimesSubsetAll(indicesToConsider,
                                          w, compatibility)
            comp_varInds = indicesToConsider[np.where(abs(
                                    compatibility[indicesToConsider]) <
                                    self.EPSILON)[0]]

            comp_rc = abs(rc[comp_varInds])
            if len(comp_rc) > 0:
                if True:
                    s = self.clpModel
                    cl = self.complementarityList
                    maxCompRc = 0
                    count = 0
                    #randinds = np.random.randint(len(comp_varInds), size=10)
                    nn = 15
#                    for i in range(random.randint(0,
#                                    len(comp_varInds)),
#                                    min(nn, len(comp_varInds))):
                    for i in range(min(nn, len(comp_varInds))):
                    #for i in randinds:
                        ind = comp_varInds[i]
                        if (s.CLP_getVarStatus(cl[ind]) != 1 and
                                   comp_rc[i] > maxCompRc):
                            maxCompInd = ind
                            maxCompRc = rc[maxCompInd]
                else:
                    maxCompInd = comp_varInds[np.argmax(comp_rc)]
                    maxCompRc = rc[maxCompInd]
            del compatibility
            del comp_rc
            del comp_varInds

        if len(rc2) > 0:
            maxInd = indicesToConsider[np.argmax(rc2)]
            maxRc = rc[maxInd]

        del rc2

        if maxCompInd != -1 and abs(maxCompRc) > 0.00001 * abs(maxRc):
            self.compCount += 1
            #print(s.getVarNameByIndex(maxCompInd))
            return maxCompInd

        if self.iCounter % self.iInterval == 0:
            rhs = self.rhs
            s.getRightHandSide(rhs)
            p_count = len(np.where(np.abs(rhs) > self.EPSILON)[0])
            if abs(p_count - self.last_p_count) > 10:
                #print(s.iteration)
                self.iCounter = 0
                self.updateW()
                self.last_p_count = p_count
                self.iInterval = max(50, self.iInterval - 50)
            else:
                self.iInterval = min(300, self.iInterval + 50)
                self.iCounter += 1
        else:
            self.iCounter += 1
        self.nonCompCount += 1
        self.CompIter = False
        #if self.numberOfIncompSinceLastUpdate > 5:
        #    self.updateW()
        #    self.numberOfIncompSinceLastUpdate = 0
        #else:
        #    self.numberOfIncompSinceLastUpdate += 1

#
#        for i in range(s.nConstraints):
#            print(s.getVarNameByIndex(s.getPivotVariable()[i]),)
#        print()

        return maxInd

#
#
#        if rc2.shape[0] > 0:
#            if checkFree:
#                w = np.where(s.varIsFree)[0]
#                if w.shape[0] > 0:
#                    ind = s.argWeightedMax(rc2, indicesToConsider, 1, w)
#                else:
#                    ind = np.argmax(rc2)
#            else:
#                    ind = np.argmax(rc2)
#            #print('incomming var: %d' % indicesToConsider[ind])
#            ret = indicesToConsider[ind]
#            del indicesToConsider  # not sure if this is necessary
#            del rc2  # HUGE memory leak otherwise
#            return ret
#        return -1
#        return self.pivotColumnFirst()

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
        #import pdb; pdb.set_trace()
#        return True
#        #TODO ComplementarityList can be defined in the current class
        s = self.clpModel
        #cl = s.getComplementarityList()
        cl = self.complementarityList
        pivotRow = s.pivotRow()
        if pivotRow < 0:
            colInd = s.sequenceIn()
            #print('entering: ', colInd, ' comp: ', cl[colInd])
            return 1

        pivotVariable = s.getPivotVariable()
        leavingVarIndex = pivotVariable[pivotRow]
        colInd = s.sequenceIn()

#        print('Basis:'))
#        print(s.getPivotVariable())
#        print('leave: ', leavingVarIndex)
#        print('entering: ', colInd, ' comp: ', cl[colInd])

        if s.CLP_getVarStatus(cl[colInd]) == 1 and \
            cl[colInd] != leavingVarIndex:
            #self.banList = np.concatenate((self.banList, [colInd]))
            self.notBanned[colInd] = False
            #s.setFlagged(colInd)
            if self.CompIter:
                self.compRej += 1
            return 0


        #self.banList = np.zeros(self.dim, np.int)
        del self.notBanned
        self.notBanned = self.orgBan.copy()
        #self.notBanned = np.array(self.dim * [True])

        return 1

    def setComplement(self, model, v1, v2):
        v1n = v1.name
        v2n = v2.name
        listv1 = np.array(model.inds.varIndex[v1n])[v1.indices]
        listv2 = np.array(model.inds.varIndex[v2n])[v2.indices]
        for i, j in zip(listv1, listv2):
            (self.complementarityList[i], self.complementarityList[j]) = \
             (self.complementarityList[j], self.complementarityList[i])




# cython: embedsignature=True

cdef class CyCbcNode:
    def __cinit__(self):
        self.CppSelf = new CppICbcNode()

    cdef setCppSelf(self, CppICbcNode* cbcnode):
        self.CppSelf = cbcnode
        return self

    property depth:
        def __get__(self):
            return self.CppSelf.depth()

    property numberUnsatisfied:
        def __get__(self):
            return self.CppSelf.numberUnsatisfied()

    property sumInfeasibilities:
        def __get__(self):
            return self.CppSelf.sumInfeasibilities()

    property active:
        def __get__(self):
            return self.CppSelf.active()

    property onTree:
        def __get__(self):
            return self.CppSelf.onTree()

    property objectiveValue:
        def __get__(self):
            return self.CppSelf.objectiveValue()

    def breakTie(self, CyCbcNode y):
        return self.CppSelf.breakTie(y.CppSelf)



# cython: profile=True
# cython: embedsignature=True

np.import_array()

cimport cylp.cy.CyClpDualRowPivotBase
#from cylp.cy import CyClpSimplex

cdef int RunPivotRow(void *ptr):
    return (<CyClpDualRowPivotBase>(ptr)).pivotRow()

cdef CyClpDualRowPivot* RunDualPivotClone(void *ptr, bint copyData):
    return (<CyClpDualRowPivotBase>(ptr)).clone(copyData)

cdef double RunUpdateWeights(void *ptr,CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn):
    return (<CyClpDualRowPivotBase>(ptr)).updateWeights(inp, spare, spare2, updatedColumn)

cdef void RunUpdatePrimalSolution(void *ptr,CppCoinIndexedVector * inp,
                                       double theta,
                                       double * changeInObjective):
    cdef np.npy_intp shape = <np.npy_intp> 1
    cho = np.PyArray_SimpleNewFromData(1, &shape, np.NPY_DOUBLE, <void*>changeInObjective)
    (<CyClpDualRowPivotBase>(ptr)).updatePrimalSolution(inp, theta, cho)


cdef class CyClpDualRowPivotBase:
    def __init__(self):
        self.CppSelf = new CppClpDualRowPivotBase(
            <cpy_ref.PyObject*>self,
            <runPivotRow_t>RunPivotRow,
            <runDualPivotClone_t>RunDualPivotClone,
            <runUpdateWeights_t>RunUpdateWeights,
            <runUpdatePrimalSolution_t>RunUpdatePrimalSolution)

    cdef pivotRow(self):
        raise Exception('CyClpDualRowPivotBase.pyx: pivotRow() should' \
                        ' be implemented.')

    cdef CyClpDualRowPivot* clone(self, bint copyData):
        cdef CyClpDualRowPivot* ret =  \
                <CyClpDualRowPivot*> new CppClpDualRowPivotBase(
                                            <cpy_ref.PyObject*>self,
                                            <runPivotRow_t>RunPivotRow,
                                            <runDualPivotClone_t>RunDualPivotClone,
                                            <runUpdateWeights_t>RunUpdateWeights,
                                            <runUpdatePrimalSolution_t>RunUpdatePrimalSolution)
        return ret

    cdef double updateWeights(self, CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn):
        raise Exception('CyClpDualRowPivotBase.pyx: updateWeights should ' \
                        'be implemented.')

    cdef void updatePrimalSolution(self, CppCoinIndexedVector * inp,
                                       double theta,
                                       np.ndarray[np.double_t,ndim=1] changeInObjective):
        raise Exception('CyClpDualRowPivotBase.pyx: updatePrimalSolution should ' \
                        'be implemented.')


    cdef CyClpSimplex.CppIClpSimplex* model(self):
        return self.CppSelf.model()

    cdef void setModel(self, CyClpSimplex.CppIClpSimplex* m):
        self.CppSelf.setModel(m)

    cdef double* getReducedCosts(self):
        return self.model().djRegion()

    property nRows:
        def __get__(self):
            return self.CppSelf.model().getNumRows()

    property nCols:
        def __get__(self):
            return self.CppSelf.model().getNumCols()



from cpython.ref cimport PyObject

cdef extern from "ICoinPackedMatrix.hpp":
    cdef cppclass CppCoinPackedMatrix "ICoinPackedMatrix":
        int getNumElements()
        PyObject* np_getIndices()
        PyObject* np_getElements()
        PyObject* np_getVectorStarts()
        int getMajorDim()
        int getMinorDim()
        void reserve(int newMaxMajorDim, int newMaxSize, bint create)
        CppCoinPackedMatrix(bint colordered, int* rowIndices,
                                    int* colIndices, double* elements,
                                    int numels)
        void appendRow(int vecsize, int *vecind, double *vecelem)
        void appendCol(int vecsize, int *vecind, double *vecelem)
        bint isColOrdered()
        void dumpMatrix(char* fname)
        PyObject * np_getMajorIndices()
        bint hasGaps()
        void removeGaps(double removeValue)
        #void setItem(int n, double value)
        #void reserve(int n)
    CppCoinPackedMatrix *new_CppCoinPackedMatrix "new ICoinPackedMatrix" ()


cdef class CyCoinPackedMatrix:
    cdef CppCoinPackedMatrix* CppSelf
    #cpdef getIndices(self)
    #cpdef getElements(self)
    #cpdef getNumElements(self)
    #cpdef getVectorStarts(self)
    #cpdef getMajorDim(self)
    #cpdef getMinorDim(self)



'''
As a part of ``cylp.python.pivots`` it implements the positive edge
pivot selection rule.
'''

import random
import numpy as np
from cylp.cy import CyCoinIndexedVector
from .PivotPythonBase import PivotPythonBase


class PositiveEdgePivot(PivotPythonBase):
    '''
    Positive Edge pivot rule implementation.

    .. _custom-pivot-usage:

    **Usage**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.pivots import PositiveEdgePivot
    >>> from cylp.py.pivots.PositiveEdgePivot import getMpsExample
    >>> # Get the path to a sample mps file
    >>> f = getMpsExample()
    >>> s = CyClpSimplex()
    >>> s.readMps(f)  # Returns 0 if OK
    0
    >>> pivot = PositiveEdgePivot(s)
    >>> s.setPivotMethod(pivot)
    >>> s.primal()
    'optimal'
    >>> round(s.objectiveValue, 5)
    2520.57174

    '''

    def __init__(self, clpModel, EPSILON=10 ** (-7)):
        self.clpModel = clpModel
        self.dim = self.clpModel.nRows + self.clpModel.nCols

        self.isDegenerate = False

        # Create some numpy arrays here ONCE to prevent memory
        # allocation at each iteration
        self.aColumn = CyCoinIndexedVector()
        self.aColumn.reserve(self.dim)
        self.w = CyCoinIndexedVector()
        self.w.reserve(self.clpModel.nRows)

        self.rhs = np.empty(self.clpModel.nRows, dtype=np.double)
        self.EPSILON = EPSILON
        self.lastUpdateIteration = 0

    def updateP(self):
        '''Finds constraints with abs(rhs) <=  epsilon and put
        their indices in "z"
        '''
        s = self.clpModel
        nRows = s.nRows

        rhs = self.rhs
        s.getRightHandSide(rhs)

        #self.p = np.where(np.abs(rhs) > self.EPSILON)[0]
        self.z = np.where(np.abs(rhs) <= self.EPSILON)[0]

        #print('degeneracy level : ', (len(self.z)) / float(nRows)))
        self.isDegenerate = (len(self.z) > 0)

    def updateW(self):
        '''Sets "w" to be a vector of random vars with "0"
        at indices defined in "p"
        Note that vectorTimesB_1 changes "w"
        '''
        self.updateP()
        self.w.clear()
        self.w[self.z] = np.random.random(len(self.z))
        s = self.clpModel
        s.vectorTimesB_1(self.w)

        self.lastUpdateIteration = s.iteration

    def random(self):
        'Defines how random vector "w" components are generated'
        return random.random()

    def isCompatible(self, varInd):
        if not self.isDegenerate:
            return False
        s = self.clpModel
        s.getACol(varInd, self.aColumn)

        return abs(cylp.cy.CyClpSimplex.cydot(self.aColumn, self.w)) < self.EPSILON

    def checkVar(self, i):
        return self.isCompatible(i)

    def pivotColumn(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        self.updateReducedCosts(updates, spareRow1, spareRow2, spareCol1, spareCol2)
        s = self.clpModel
        rc = s.reducedCosts

        tol = s.dualTolerance
        indicesToConsider = np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree))[0]

        rc2 = abs(rc[indicesToConsider])

        maxRc = maxCompRc = maxInd = maxCompInd = -1

        if self.isDegenerate:
            w = self.w.elements
            compatibility = np.zeros(s.nCols + s.nRows, dtype=np.double)
            if len(indicesToConsider) > 0:
                s.transposeTimesSubsetAll(indicesToConsider.astype(np.int64),
                                          w, compatibility)
            comp_varInds = indicesToConsider[np.where(abs(
                                    compatibility[indicesToConsider]) <
                                    self.EPSILON)[0]]

            comp_rc = abs(rc[comp_varInds])
            if len(comp_rc) > 0:
                maxCompInd = comp_varInds[np.argmax(comp_rc)]
                maxCompRc = rc[maxCompInd]

        if len(rc2) > 0:
            maxInd = indicesToConsider[np.argmax(rc2)]
            maxRc = rc[maxInd]

        del rc2
        if maxCompInd != -1 and abs(maxCompRc) > 0.1 * abs(maxRc):
            return maxCompInd
        self.updateW()
        return maxInd

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
        return True


def getMpsExample():
    import os
    import inspect
    cylpDir = os.environ['CYLP_SOURCE_DIR']
    return os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')




class cylpCutGenerator:
    '''
    Abstract base class
    '''
    def generateCuts(self, x, y, z):
        '''
        **Abstract Method**

        Return True if ``y`` is better than ``x``
        :arg y: A B&B node
        :type y: CyCbcNode
        :rtype: list
        '''
        raise NotImplementedError('cylpCutGenerator object must ' \
                        'implement *generateCuts*')




cdef class CyCglTreeInfo:
    'CyCglTreeInfo documentation'
    def __cinit__(self):
        self.CppSelf = new CppCglTreeInfo()

    cdef setCppSelf(self, CppCglTreeInfo* s):
        del self.CppSelf
        self.CppSelf = s




import inspect
import os
import unittest
import numpy as np
from cylp.cy import CyClpSimplex
from cylp.py.utils.sparseUtil import csr_matrixPlus
from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray, getCoinInfinity

inf = getCoinInfinity()
currentFilePath = os.path.dirname(inspect.getfile(inspect.currentframe()))

class TestModeling(unittest.TestCase):

    def setUp(self):
        self.model = CyLPModel()
        model = self.model

        self.x = model.addVariable('x', 5)


        self.b = np.array([3.1, 4.2])

        self.A = np.matrix([[1, 2, 3, 3, 4], [3, 2, 1, 2, 5]])
        self.B = np.matrix([[0, 0, 0, -1.5, -1.5], [-3, 0, 0, -2,0 ]])
        self.D = np.matrix([[1, 3, 1, 4], [2, 1,3, 5]])

    def test_Obj1(self):
        m = self.model
        x = self.x
        y = m.addVariable('y', 4)
        z = m.addVariable('z', 5)
        A = self.A
        b = self.b

        k = m.addVariable('k', 2)

        m.addConstraint(x >= 0)

        m.addConstraint(A * x <= b)

        m.addConstraint(z >= 0)

        m.objective = x.sum() + z.sum() + k.sum()

    def test_variableBoundSubset(self):
        m = self.model
        x = self.x
        y = m.addVariable('y', 4)
        z = m.addVariable('z', 5)
        A = self.A
        b = self.b

        k = m.addVariable('k', 2)

        s = CyClpSimplex(m)

        s.setColumnLowerSubset(np.array([1, 2], np.int32), np.array([3, 5, 8],
                                        np.int32), np.array([3.2, 3.1, 2.2]))
        self.assertTrue(s.variablesLower[3] != 3.2)
        self.assertTrue(s.variablesLower[5] == 3.1)
        self.assertTrue(s.variablesLower[8] == 2.2)

        s.setColumnUpperSubset(np.array([0, 2], np.int32), np.array([0, 4, 10],
                              np.int32), np.array([3.2, 3.1, 2.2]))
        self.assertTrue(s.variablesUpper[0] == 3.2)
        self.assertTrue(s.variablesUpper[4] != 3.1)
        self.assertTrue(s.variablesUpper[10] == 2.2)



    def test_bound1(self):
        model = self.model
        x = self.x

        model.addConstraint(x[2:4] >= 3)
        self.assertTrue((x.lower == np.array([-inf, -inf, 3, 3, -inf])).all())
        self.assertTrue((x.upper == np.array(5* [inf])).all())

    def test_bound2(self):
        model = self.model
        x = self.x

        model.addConstraint(x[2:4] <= 3)
        self.assertTrue((x.lower == np.array(5 * [-inf])).all())
        self.assertTrue((x.upper == np.array([inf, inf, 3, 3, inf])).all())

    def test_bound3(self):
        model = self.model
        x = self.x
        b = self.b

        model.addConstraint(1.1 <= x[2:4] <= b)
        self.assertTrue((x.lower == np.array(
                            [-inf, -inf, 1.1, 1.1, -inf])).all())
        self.assertTrue((x.upper == np.array(
                            [inf, inf, b[0], b[1], inf])).all())
        m, cl, cu, vl, vu = model.makeMatrices()
        self.assertTrue((vu[:5] == np.array(
                                    [inf, inf, b[0], b[1], inf])).all())
        self.assertTrue((vl[:5] == np.array(
                                    [-inf, -inf, 1.1, 1.1, -inf])).all())

    def test_bound4(self):
        model = self.model
        x = self.x
        b = self.b

        model += x[2:4] == b
        m, cl, cu, vl, vu = model.makeMatrices()
        self.assertTrue((vu[:5] == np.array(
                                    [inf, inf, b[0], b[1], inf])).all())
        self.assertTrue((vl[:5] == np.array(
                                    [-inf, -inf, b[0], b[1], -inf])).all())

    def test_constraint_single1(self):
        model = self.model
        x = self.x

        model += 1.1 <= x[0] - 3 * x[1:3].sum() + 2 * x[2:5].sum() <= 4.5
        cons = model.constraints[0]
        m, cl, cu, vl, vu = model.makeMatrices()
        self.assertTrue((m.todense() == np.array(
                            [1, -3, -1, 2, 2])).all())

    def test_constraint_single2(self):
        model = self.model
        x = self.x

        model.addConstraint(-x[1] + 3 * x[1:4].sum() + None * x >= 4.5)
        cons = model.constraints[0]
        m, cl, cu, vl, vu = model.makeMatrices()
        self.assertTrue((m.todense() == np.array(
                            [0, 2, 3, 3, 0])).all())
        self.assertTrue((cons.lower == np.array(
                            [4.5])).all())
        self.assertTrue((cons.upper == np.array(
                            [inf])).all())


    def test_constraint_single3(self):
        model = self.model
        x = self.x


        k = csr_matrixPlus([[3.1], [4.2]])
        model.addConstraint(-x[0] + -3 * x[1] + k.T * x[1:3]  >= 4.5)
        cons = model.constraints[0]
        m, cl, cu, vl, vu  = model.makeMatrices()
        self.assertTrue((abs(m.todense()-np.array(
                            [-1, 0.1, 4.2, 0, 0])) < 0.000001).all())

    def test_constraint_1(self):
        model = self.model
        x = self.x
        b = self.b

        model.addConstraint(2 <= -x <= 4.5)

        cons = model.constraints[0]

        m, cl, cu, vl, vu = model.makeMatrices()
        self.assertTrue((abs(m.todense() +
                        np.eye(5, 5)) < 0.000001).all())

        #self.assertEqual(self.c[4], 3.3)

    def test_removeConst(self):

        m = self.model
        x = self.x
        A = self.A
        b = self.b


        m.addConstraint(x >= 0)

        m.addConstraint(A * x == b)

        m.addConstraint(x[1:3].sum() >= 1, 'rr')

        m.objective = x.sum()
        s = CyClpSimplex(m)
        s.primal()
        self.assertAlmostEqual(s.primalVariableSolution['x'][1], 1, 7)

        s.removeConstraint('rr')
        s.primal()
        self.assertAlmostEqual(s.primalVariableSolution['x'][1], 0, 7)

    def test_removeVar(self):
        m = self.model
        x = self.x
        A = self.A
        B = self.B
        D = self.D
        b = self.b

        y = m.addVariable('y', 4)
        z = m.addVariable('z', 5)

        m.addConstraint(x >= 0)
        m.addConstraint(y >= -10)
        m.addConstraint(z >= -10)

        m.addConstraint(A * x + D * y + B * z <= b)
        m += x[0] + y[0] + z[0] >= 1.12

        m.objective = x.sum() + y.sum() + z.sum()
        s = CyClpSimplex(m)
        s.primal()

        self.assertTrue('y' in s.primalVariableSolution.keys())

        m.removeVariable('y')
        s = CyClpSimplex(m)
        s.primal()
        self.assertTrue('y' not in s.primalVariableSolution.keys())

    def test_removeVar2(self):
        s = CyClpSimplex()
        fp = os.path.join(currentFilePath, '../../input/p0033.mps')
        s.extractCyLPModel(fp)
        y = s.addVariable('y', 3)
        s.primal()

        x = s.getVarByName('x')
        s.addConstraint(x[1] +  y[1] >= 1.2)
        #s.primal()
        s.removeVariable('x')
        s.primal()
        s = s.primalVariableSolution
        self.assertTrue((s['y'] - np.array([0, 1.2, 0]) <= 10**-6).all())


if __name__ == '__main__':
    unittest.main()




from cylp.py import Constants
from cylp.py.QP import QP
from cylp.py.utils import util
import numpy as np
#import ipdbc


class GQP(QP.QP):
    '''
    GQP is a QP that is prepared to be solved using one of
    Goldfarb's primal or dual methods.
    '''
    def __init__(self, L, a, CT, b, ne=0):
        QP.QP.__init__(self, L, a, CT, b)
        # list of integers, the working set
        self.A = []
        # J = L^{-T}Q
        self.J = self.L.T.I
        # vector, dual variables
        self.u = None
        ## L^{-1}N = [Q_1 | Q_2] [R | 0]^T
        self.R = None
        # number of equality variables, the coefficients are
        # in the first ne rows of CT
        self.ne = ne
        self.numberOfActiveSetChanges = 0

        self.x = self.getUnconstrainedSol()
        self.f = 0.5 * self.a.T * self.x
        if ne == 0:
            self.x = np.matrix(self.x, Constants.DATATYPE)

        for i in range(ne):
            n_new = self.CT[i, :].T
            z = self.J2 * self.J2.T * n_new

            s = -self.sSome(self.x, [i])
            t2 = 0
            if (z.T * n_new) >= Constants.EPSILON:
                t2 = (s / (z.T * n_new))[0, 0]

            self.x += t2 * z
            self.f += 0.5 * (t2 * t2) * (z.T * n_new)

            try:
                self.u = np.concatenate((self.u, np.matrix([[t2]])))
            except:
                self.u = np.matrix([[t2]])

            if self.q > 0:
                r = np.linalg.solve(self.R, self.getd1(i))
                self.u[:self.q] -= t2 * r
            self.addConstraint(i)

    def sSome(self, x, subsetOfCons):
        m = [self.CT[i, :] * x - self.b[i] for i in subsetOfCons]
        return np.concatenate(m)

    def getq(self):
        '''
        getter for property q, the number of active constraints
        '''
        return len(self.A)

    q = property(getq)

    def getMostViolatedConst(self, x):
        '''
        returns the index of the most violated constraint by the point x
        '''
        s = self.sAll(x)

        # Set the slack of the active constraints to
        # exact zero to make sure they are never selected
        for i in range(self.m):
            if i in self.A:
                s[i] = 0
        minInd = s.argmin()
        if s[minInd] > -Constants.EPSILON:
            return -1
        return s.argmin()

    def getJ1(self):
        '''
        Return J_1 the first q columns of #J as a n*q matrix
        '''
        return self.J[:, :self.q]

    J1 = property(getJ1)

    def getJ2(self):
        '''
        returns J_2, the last n-q columns of #J as a n*(n-q) matrix
        '''
        return self.J[:, self.q:]

    J2 = property(getJ2)

    def getd1(self, i):
        return self.J1.T * self.CT[i, :].T

    d1 = property(getd1)

    def getd2(self, i):
        return self.J2.T * self.CT[i, :].T

    d2 = property(getd2)

    def getDualDirect_Step(self, r):
        '''
        - append a 1 to the end of -r to get the dual direction (DAS)
        - If the working set is empty, returns a vector with a
        single element = 1
        '''
        if self.q > 0:
            return np.concatenate((-r, np.matrix([[1.]])))
        return np.matrix([[1]])

    def checkInvars(self):
        ret = True
        if self.q > 0:
            Q1, R = np.linalg.qr(self.L.I *
                                (self.CT[[i for i in self.A], :]).T)

            # Make sure that Q2 and J2 are alright
            if (abs((self.L.T * self.J2).T * Q1) > 0.1).any() or \
                (abs(Q1.T * (self.L.T * self.J2)) > 0.1).any():
                #print("post1" , abs((self.L.T * self.J2).T * Q1))
                #print(abs(Q1.T * (self.L.T * self.J2)))
                #print("--")
                ret = False

            # Make sure that R is OK and N_star IS the pseudo-inverse of N
            N_star_N = (R.I * self.J1.T) * (self.CT[[i for i in self.A], :].T)
            iden = np.matrix(np.identity(self.q, float))

            if (abs(N_star_N) - iden > 0.5).any():
                #print("post2" , abs(N_star_N) - iden)
                #print("--")
                ret = False
        return ret

    def addConstraint(self, i, d1=None, d2=None):
        '''
        Adds constraint i to the working set A by:
            - Using the Givens transformations to update J and R
            - Appending index i to the end of A
        '''
        try:
            d1.shape
        except:
            d1 = self.getd1(i)
            d2 = self.getd2(i)
        Ql, Q_bar, delta = util.applyGivens(d2)
        self.J = np.concatenate((self.J1, self.J2 * Q_bar.T), axis=1)
        if self.q == 0:
            self.R = np.matrix([[delta]], Constants.DATATYPE)
        else:
            self.R = np.concatenate((self.R, d1), axis=1)
            lastline = np.matrix([[0 for k in range(self.q)] + [delta]])
            self.R = np.concatenate((self.R, lastline))

        self.A.append(i)
        self.numberOfActiveSetChanges += 1
        return Q_bar

    def removeConstraint(self, l, uPlus):
        '''Removes constraint number l from the working set #A:
             -Removes the corresponding dual variable from uPlus
             -Updates J and R
             -Removes the l'th element of A
        '''
        q = self.q
        if l == 0:
            uPlus = uPlus[1:, 0]
        else:
            u_l = uPlus[l, 0]
            uPlus = np.concatenate((uPlus[:l, 0], uPlus[(l + 1):, 0]))

        if q == 1:
            self.R = None
            #TODO: Verify
            #self.J = self.L.T.I
        elif l == q - 1:
            self.R = self.R[:(q - 1), :(q - 1)]
        else:
            T = self.R[l:, (l + 1):]
            Q_bar, R2 = util.UH2UT(T)
            if l == 0:
                self.R = R2
            else:
                #Add zeros to R2 to make the lower q-1-l rows of R_minus
                R2 = np.concatenate((np.matrix(
                                       np.zeros((q - 1 - l, l))), R2), axis=1)
                #Remove the l'th column of R
                self.R = np.concatenate((self.R[:, :l],
                                         self.R[:, (l + 1):]), axis=1)
                # Construct R_minus from the upper l rows of R
                #  and q-1-l rows of R2 constructed above
                self.R = np.concatenate((self.R[:l, :], R2))

            #constructing J_minus
            mat_temp = np.matrix(np.identity(self.n, Constants.DATATYPE))
            mat_temp[l:q, l:q] = Q_bar.T
            self.J *= mat_temp

        self.A.pop(l)
        self.numberOfActiveSetChanges += 1

        if q == 1 or l == q - 1:
            return uPlus, 1

        return uPlus, mat_temp[:q, :q]

    def minStep(self, t1, t2):
        '''
        Returns the minimum of non-negative t1 and t2
        considering -1 to denote infinity
        '''
        m = min(t1, t2)
        if m == -1:
            if t1 != -1:
                return t1
            return t2
        return m

    def getMinRatio(self, uPlus, r, sign=1):
        '''
        Return min{u_plus[j] / r[j]} for (r[j] * sign) > 0
        '''
        m = mInd = -1
        for j in range(self.ne, r.shape[0]):
            if r[j, 0] * sign > Constants.EPSILON:
                ratio = uPlus[j, 0] / r[j, 0]
                if ratio < m or m == -1:
                    m = ratio
                    mInd = j
        return m, mInd

    def dual(self, verbose):
        '''
        Goldfarb dual active-set algorithm for convex QP
        '''
        self.__init__(self.L, self.a, self.CT, self.b, self.ne)

        iteration = 0
        while True:
            iteration += 1
            if verbose:
                print('-----------------------------------------')
                print('iteration = ', iteration)

                #print("obj value=" , f)
                print("active set:")
                print(self.A)
            p = self.getMostViolatedConst(self.x)

            if p == -1:
                return 'feasible', self.x, self.f[0, 0]

            if self.q == 0:
                self.u = np.matrix([[0.]], Constants.DATATYPE)
                uPlus = np.matrix([[0.]], Constants.DATATYPE)
            else:
                uPlus = np.concatenate((self.u, np.matrix([[0.]])))

            n_new = self.CT[p, :].T

            #step2: partial step
            while True:
                #step 2.a)
                z = self.J2 * self.J2.T * n_new
                dual_dir = None
                if self.q > 0:
                    'calculate r'
                    r = np.linalg.solve(self.R, self.getd1(p))
                    dual_dir = np.concatenate((-r, np.matrix([[1.]],
                                              Constants.DATATYPE)))
                else:
                    dual_dir = np.matrix([[1]], Constants.DATATYPE)

                #step 2.b)
                t1 = t2 = -1
                #Partial Step Length
                if self.q > 0 and (r > 0).any():
                    t1, k = self.getMinRatio(uPlus, r)

                #Full Step Length
                if (abs(z) > Constants.EPSILON).any():
                    t2 = (-self.sSome(self.x, [p]) / (z.T * n_new))[0, 0]

                t = self.minStep(t1, t2)

                #step 2.c)
                if t == -1:
                    return 'infeasible', -1, -1
                elif t2 == -1:
                    if verbose:
                        print("taking a partial step_1")
                    uPlus += t * dual_dir
                    #we'll not use Q_tilde here. It is used in the PAS method.
                    uPlus, Q_tilde = self.removeConstraint(k, uPlus)
                    continue
                self.x += t * z

                #Not sure if it's smart to update f at each iteration
                self.f += t * z.T * n_new * (0.5 * t + uPlus[-1, 0])
                uPlus += t * dual_dir

                if abs(t - t2) <= Constants.EPSILON:
                    if verbose:
                        print("taking a full step ")
                        print('blocking constriant : ', p)
                    self.u = uPlus
                    self.addConstraint(p)
                    #We took the full step, going back to step 1
                    break
                else:
                    if verbose:
                        print("taking a partial step_2")
                    #t == t1
                    uPlus, Q_tilde = self.removeConstraint(k, uPlus)
                    continue

    def primal(self, x, A, verbose):
        '''
        Goldfarb Primal active-set algorithm for convex QP
        '''
        for activeConstraint in A:
            self.addConstraint(activeConstraint)

        u = np.linalg.solve(self.R, self.J1.T * self.gradient(x))

        iteration = 0
        while True:
            iteration += 1
            if self.q > 0:
                p = u.argmin()
                IndOfConstraintToBeRemoved = self.A[p]
                if u[p] >= -Constants.EPSILON:
                    return 1, x, 0

                u_p = u[p, 0]
                R_p = self.R[:, p]

                u, Q_tilde = self.removeConstraint(p, u)

                d = Q_tilde * (u_p * R_p)

                d1 = d[:-1, 0]
                d2 = np.zeros((self.n - self.q, 1))
                d2[0, 0] = d[-1, 0]

            while self.q < self.n:

                #2
                if self.q < self.n:
                    z = -self.J2 * d2
                else:
                    z = 0

                if self.q > 0:
                    r = np.linalg.solve(self.R, d1)

                t1 = 1

                nonActive = [k for k in range(self.m) if k not in self.A]
                v = np.concatenate([self.CT[i, :] * z for i in nonActive], 1).T
                s = self.sSome(x, nonActive)

                m, mInd = self.getMinRatio(-s, v, -1)

                t1 = 1
                if m != -1:
                    t1 = min(1, m)

                x += t1 * z
                if self.q > 0:
                    u += t1 * r

                if t1 != 1:
                    Q_bar = self.addConstraint(nonActive[mInd])
                    u = np.concatenate((u, np.matrix([[0.]])))
                    d22 = self.J2.T * self.CT[0, :].T
                    d = (1 - t1) * d
                    d1 = d[:(self.q), 0]
                    d2 = d[(self.q):, 0]
                else:
                    break

        return 0, 0, 0



import sys
from time import perf_counter
from cylp.cy.CyClpSimplex cimport CyClpSimplex
from cylp.cy.CyDantzigPivot cimport CyDantzigPivot
from cylp.cy.CyPEPivot cimport CyPEPivot


def CySolve(fileName, method):
    cdef CyClpSimplex s = CyClpSimplex()
    s.readMps(fileName, 0, 0)

    cdef CyDantzigPivot dpivot
    cdef CyPEPivot ppivot

    if method == 'd':
        dpivot = CyDantzigPivot(s)
        #s.setPerturbation(50)
        s.setPrimalColumnPivotAlgorithm(dpivot.CppSelf)
    elif method == 'p':
        ppivot = CyPEPivot(s)
        s.setPrimalColumnPivotAlgorithm(ppivot.CppSelf)

    start = perf_counter()
    s.primal()
    print('Exec time: ',  perf_counter() - start)
    return s.objectiveValue



import os
import math
import numpy as np
from cylp.cy import CyClpSimplex
from cylp.py.modeling import CyLPArray, CyLPModel
from cylp.py.modeling.CyLPModel import getCoinInfinity

epsilon = 0.01

def isInt(x):
    '''
    Return True if x is an integer, or if x is a numpy array
    with all integer elements, False otherwise
    '''
    if isinstance(x, (int, float)):
        return abs(math.floor(x) - x) < epsilon
    return (np.abs(np.floor(x) - x) < epsilon).all()

def gomoryCut(lp, rowInd):
    'Return the Gomory cut of row ``rowInd`` of lp (a CyClpSimplex object)'
    fractions = np.array([getFraction(x) for x in lp.tableau[rowInd, :]])
    pi, pi0 = fractions[:lp.nVariables], getFraction(lp.rhs[rowInd])
    pi_slacks = fractions[lp.nVariables:]
    pi -= pi_slacks * lp.coefMatrix
    pi0 -= np.dot(pi_slacks, lp.constraintsUpper)
    if (abs(pi) > 1e-6).any():
        return pi, pi0
    return None, None

def getFraction(x):
    'Return the fraction part of x: x - floor(x)'
    return x - math.floor(x)

class GomoryCutGenerator:
    def __init__(self, cyLPModel):
        self.cyLPModel = cyLPModel

    def generateCuts(self, si, cglTreeInfo):
        m = self.cyLPModel
        x = m.getVarByName('x')

        clpModel = si.clpModel
        clpModel.dual(startFinishOptions='x')
        solution = clpModel.primalVariableSolution
        bv = clpModel.basicVariables
        rhs = clpModel.rhs

        intInds = clpModel.integerInformation

        rhsIsInt = map(isInt, rhs)

        cuts = []
        for rowInd in range(s.nConstraints):
            basicVarInd = bv[rowInd]
            if basicVarInd < clpModel.nVariables and intInds[basicVarInd] and not rhsIsInt[rowInd]:
                coef, b = gomoryCut(clpModel, rowInd)
                if b is not None:
                    #print('Adding cut: ', coef, '>=', b)
                    expr = CyLPArray(coef) * x >= b
                    cuts.append(expr)
        return cuts


if __name__ == '__main__':
    m = CyLPModel()

    firstExample = False

    if (firstExample):
        x = m.addVariable('x', 2, isInt=True)

        A = np.matrix([[7., -2.],[0., 1], [2., -2]])
        b = CyLPArray([14, 3, 3])

        m += A * x <= b
        m += x >= 0

        c = CyLPArray([-4, 1])
        m.objective = c * x
        s = CyClpSimplex(m)
    else:
        s = CyClpSimplex()
        cylpDir = os.environ['CYLP_SOURCE_DIR']
        inputFile = os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')
        m = s.extractCyLPModel(inputFile)
        x = m.getVarByName('x')
        s.setInteger(x)

    cbcModel = s.getCbcModel()

    gc = GomoryCutGenerator(m)
    cbcModel.addPythonCutGenerator(gc, name='PyGomory')

    cbcModel.branchAndBound()
    cbcModel.solve()

    print(cbcModel.primalVariableSolution)



cimport numpy as np

cdef extern from "CoinModel.hpp":
    cdef cppclass CppCoinModel "CoinModel":

        void addRow(int numberInRow, int * columns,
                        double * elements, double rowLower,
                        double rowUpper, char * name)

        void addRow(int numberInRow, int * columns,
                        double * elements, double rowLower,
                        double rowUpper)

        void addColumn(int numberInColumn, int * rows,
                double * elements,
                double columnLower,
                double columnUpper, double objectiveValue,
                char * name, int isInteger)

        void addColumn(int numberInColumn, int * rows,
                double * elements,
                double columnLower,
                double columnUpper, double objectiveValue)

        void setColumnLower(int whichColumn, double columnLower)
        void setColumnUpper(int whichColumn, double columnLower)
        void setRowLower(int whichColumn, double columnLower)
        void setRowUpper(int whichColumn, double columnLower)
        void setObjective(int whichColumn, double columnObjective)
        int numberRows()
        int numberColumns()


cdef class CyCoinModel:
    cdef CppCoinModel* CppSelf

#   cdef void CLP_addColumn(self, int numberInColumn,
#                                   int * rows,
#                                   double * elements,
#                                   double columnLower,
#                                   double columnUpper,
#                                   double objectiveValue,
#                                   char * name,
#                                   int isInteger)
#
    cdef void CLP_addColumn(self, int numberInColumn,
                            int * rows,
                            double * elements,
                            double columnLower,
                            double columnUpper,
                            double objectiveValue)

#   cdef void CLP_addRow(self, int numberInRow, int * columns,
#                       double * elements, double rowLower,
#                       double rowUpper, char * name)

    cdef void CLP_addRow(self, int numberInRow, int * columns,
                        double * elements, double rowLower,
                        double rowUpper)



import unittest
import inspect
import os
from os.path import join
import numpy as np
from cylp.cy import CyClpSimplex
from .PySolve import solve


problem = os.path.join(os.path.dirname(__file__), '..', 'input',
                       'netlib', 'adlittle.mps')

adlittleSol = 225494.963162

class TestPySolve(unittest.TestCase):

    def test_pe(self):
        objval = solve(problem, 'p')
        self.assertAlmostEqual(objval, adlittleSol, 6)

    def test_dantzig(self):
        objval = solve(problem, 'd')
        self.assertAlmostEqual(objval, adlittleSol, 6)

    def test_lifo(self):
        objval = solve(problem, 'l')
        self.assertAlmostEqual(objval, adlittleSol, 6)

    def test_mf(self):
        objval = solve(problem, 'm')
        self.assertAlmostEqual(objval, adlittleSol, 6)


if __name__ == '__main__':
    unittest.main()




import scipy
from scipy.sparse import csr_matrix
from cylp.py.modeling.CyLPModel import CyLPModel
cimport numpy as np
import numpy as np

cdef class CyOsiCuts:
    'CyOsiCuts documentation'
    def __cinit__(self):
        self.CppSelf = new CppOsiCuts()

    cdef setCppSelf(self, CppOsiCuts* s):
        del self.CppSelf
        self.CppSelf = s

    def printCuts(self):
        self.CppSelf.printCuts()

    property numberOfRowCuts:
        def __get__(self):
            return self.CppSelf.sizeRowCuts()

    property numberOfColumnCuts:
        def __get__(self):
            return self.CppSelf.sizeColCuts()

    property numberOfCuts:
        def __get__(self):
            return self.CppSelf.sizeCuts()

    def addColumnCut(self, cut, cyLpModel):
        '''
        Add ``cut`` to cuts. ``cut`` is a CyLPExpr
        and ``cut.isRange`` is ``True``.
        '''
        m = CyLPModel()
        for var in cyLpModel.variables:
            m.addVariable(var.name, var.dim, var.isInt)
        x = m.getVarByName('x')
        m += cut
        mat, cl, cu, vl, vu = cyLpModel.makeMatrices()

        inds = np.arange(len(vl), dtype=np.int32)
        cdef np.ndarray[np.int32_t, ndim=1] vl_inds = inds
        cdef np.ndarray[np.double_t, ndim=1] vl_data = vl
        cdef np.ndarray[np.int32_t, ndim=1] vu_inds = inds
        cdef np.ndarray[np.double_t, ndim=1] vu_data = vu

        self.CppSelf.addColumnCut(len(vl),
                                  <int*>vl_inds.data,
                                  <double*>vl_data.data,
                                  <int*>vu_inds.data,
                                  <double*>vu_data.data)

    def addRowCut(self, cut, cyLpModel):
        '''
        Add ``cut`` to cuts. ``cut`` is a CyLPExpr
        and ``cut.isRange`` is ``False``.
        '''
        m = CyLPModel()
        for var in cyLpModel.variables:
            m.addVariable(var.name, var.dim, var.isInt)
        m += cut
        mat, cl, cu, vl, vu = m.makeMatrices()

        cdef np.ndarray[np.int32_t, ndim=1] row_inds
        cdef np.ndarray[np.double_t, ndim=1] row_data

        for nr in xrange(mat.shape[0]):
            row = mat[nr, :]
            row_inds = row.indices
            row_data = row.data
            assert len(row_inds) == len(row_data)
            self.CppSelf.addRowCut(len(row_inds),
                                  <int*>row_inds.data,
                                  <double*>row_data.data,
                                  cl[nr], cu[nr])





import numpy as np
from .PivotPythonBase import PivotPythonBase


class WolfePivot(PivotPythonBase):

    def __init__(self, clpModel, bucketSize=1):
        self.dim = clpModel.nConstraints + clpModel.nVariables
        self.clpModel = clpModel
        # Tell IClpSimplex that this pivot rules needs
        #an extra check after the leaving varible is chosen.
        clpModel.useCustomPrimal(True)
        self.orgBan = np.array(self.dim * [True], np.bool)
        self.notBanned = self.orgBan.copy()
        self.complementarityList = np.arange(self.dim)


    def pivotColumn(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        self.updateReducedCosts(updates, spareRow1, spareRow2, spareCol1, spareCol2)
        s = self.clpModel

        # If objective function linear proceed as normal
#        if s.Hessian is None:
        rc = s.reducedCosts
        #tol = s.dualTolerance
        tol = 0
        indicesToConsider = np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree) &
                                     self.notBanned)[0]


        rc2 = abs(rc[indicesToConsider])

        checkFree = False
        #rc2[np.where((status & 7 == 4) | (status & 7 == 0))] *= 10
        if rc2.shape[0] > 0:
            if checkFree:
                w = np.where(s.varIsFree)[0]
                if w.shape[0] > 0:
                    ind = s.argWeightedMax(rc2, indicesToConsider, 1, w)
                else:
                    ind = np.argmax(rc2)
            else:
                    ind = np.argmax(rc2)
            ret = indicesToConsider[ind]
            del indicesToConsider  # not sure if this is necessary
            del rc2  # HUGE memory leak otherwise
            return ret
        return -1
        return self.pivotColumnFirst()

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
#        #TODO ComplementarityList can be defined in the current class
        s = self.clpModel
        #cl = s.getComplementarityList()
        cl = self.complementarityList
        pivotRow = s.pivotRow()
        if pivotRow < 0:
            colInd = s.sequenceIn()
            return 1

        pivotVariable = s.getPivotVariable()
        leavingVarIndex = pivotVariable[pivotRow]
        colInd = s.sequenceIn()

        if s.CLP_getVarStatus(cl[colInd]) == 1 and \
            cl[colInd] != leavingVarIndex:
            #print(colInd , ' flagged')
            #self.banList[colInd] = 1
            #print('banning %d' % colInd)
            #print(self.notBanned)
            #self.banList = np.concatenate((self.banList, [colInd]))
            self.notBanned[colInd] = False
            #print(self.notBanned)
            #s.setFlagged(colInd)
            return 0

        del self.notBanned
        self.notBanned = self.orgBan.copy()

        return 1

    def setComplement(self, model, v1, v2):
        v1n = v1.name
        v2n = v2.name
        listv1 = np.array(model.inds.varIndex[v1n])[v1.indices]
        listv2 = np.array(model.inds.varIndex[v2n])[v2.indices]
        for i, j in zip(listv1, listv2):
            self.complementarityList[i], self.complementarityList[j] = j, i




cimport numpy as np
from cpython.ref cimport PyObject

#############################
#Combinatorial cuts:
#############################
cdef extern from "CglAllDifferent.hpp":
    cdef cppclass CppCglAllDifferent "CglAllDifferent":
        pass

cdef extern from "CglClique.hpp":
    cdef cppclass CppCglClique "CglClique":
        pass

cdef extern from "CglKnapsackCover.hpp":
    cdef cppclass CppCglKnapsackCover "CglKnapsackCover":
        int getMaxInKnapsack()
        void setMaxInKnapsack(int)

cdef extern from "CglOddHole.hpp":
    cdef cppclass CppCglOddHole "CglOddHole":
        pass

#############################
#Flow cover cuts:
#############################

cdef extern from "CglFlowCover.hpp":
    cdef cppclass CppCglFlowCover "CglFlowCover":
        pass

#############################
#Gomory cuts and variants:
#############################

cdef extern from "CglGomory.hpp":
    cdef cppclass CppCglGomory "CglGomory":
        int getLimit()
        void setLimit(int)

cdef extern from "CglRedSplit.hpp":
    cdef cppclass CppCglRedSplit "CglRedSplit":
        pass

#############################
#Lift-and-project cuts:
#############################

cdef extern from "CglLiftAndProject.hpp":
    cdef cppclass CppCglLiftAndProject "CglLiftAndProject":
        pass

cdef extern from "CglLandP.hpp":
    cdef cppclass CppCglLandP "CglLandP":
        pass

#############################
#Mixed integer rounding cuts and variants:
#############################

cdef extern from "CglMixedIntegerRounding.hpp":
    cdef cppclass CppCglMixedIntegerRounding "CglMixedIntegerRounding":
        pass

cdef extern from "CglMixedIntegerRounding2.hpp":
    cdef cppclass CppCglMixedIntegerRounding2 "CglMixedIntegerRounding2":
        pass

cdef extern from "CglTwomir.hpp":
    cdef cppclass CppCglTwomir "CglTwomir":
        pass

cdef extern from "CglResidualCapacity.hpp":
    cdef cppclass CppCglResidualCapacity "CglResidualCapacity":
        pass

#############################
#Strengthening:
#############################

#cdef extern from "CglDuplicateRow.hpp":
#    cdef cppclass CppCglDuplicateRow "CglDuplicateRow":
#        pass

cdef extern from "CglPreProcess.hpp":
    cdef cppclass CppCglPreProcess "CglPreProcess":
        pass

cdef extern from "CglProbing.hpp":
    cdef cppclass CppCglProbing "CglProbing":
        pass

cdef extern from "CglSimpleRounding.hpp":
    cdef cppclass CppCglSimpleRounding "CglSimpleRounding":
        pass

## parent of all cuts
cdef extern from "CglCutGenerator.hpp":
    cdef cppclass CppCglCutGenerator "CglCutGenerator":
        pass


cdef class CyCglCutGenerator:
    cdef CppCglCutGenerator* CppSelf

###########

cdef class CyCglAllDifferent(CyCglCutGenerator):
    pass

cdef class CyCglClique(CyCglCutGenerator):
    pass

cdef class CyCglKnapsackCover(CyCglCutGenerator):
    cdef CppCglKnapsackCover* realCppSelf(self)

cdef class CyCglOddHole(CyCglCutGenerator):
    pass

##################

cdef class CyCglFlowCover(CyCglCutGenerator):
    pass

##################

cdef class CyCglGomory(CyCglCutGenerator):
    cdef CppCglGomory* realCppSelf(self)
    #cdef CppCglGomory* CppSelf
    #cdef CppCglCutGenerator* CppSelf

cdef class CyCglRedSplit(CyCglCutGenerator):
    pass

###################

cdef class CyCglLiftAndProject(CyCglCutGenerator):
    pass

cdef class CyCglLandP(CyCglCutGenerator):
    pass

###################

cdef class CyCglMixedIntegerRounding(CyCglCutGenerator):
    pass

cdef class CyCglMixedIntegerRounding2(CyCglCutGenerator):
    pass

cdef class CyCglTwomir(CyCglCutGenerator):
    pass

cdef class CyCglResidualCapacity(CyCglCutGenerator):
    pass

####################

#cdef class CyCglDuplicateRow(CyCglCutGenerator):
#    pass

cdef class CyCglPreProcess(CyCglCutGenerator):
    pass

cdef class CyCglProbing(CyCglCutGenerator):
    pass

cdef class CyCglSimpleRounding(CyCglCutGenerator):
    pass




# cython: embedsignature=True

cdef class CyDualPivotPythonBase(CyClpDualRowPivotBase):
    def __init__(self, dualPivotMethodObject):
        CyClpDualRowPivotBase.__init__(self)
        Py_INCREF(dualPivotMethodObject)
        self.dualPivotMethodObject = dualPivotMethodObject

    cdef pivotRow(self):
        return self.dualPivotMethodObject.pivotRow()

    cdef CyClpDualRowPivot* clone(self, bint copyData):
        cdef CyClpDualRowPivot* ret =  \
                <CyClpDualRowPivot*> new CppClpDualRowPivotBase(
                            <cpy_ref.PyObject*>self,
                            <runPivotRow_t>RunPivotRow,
                            <runDualPivotClone_t>RunDualPivotClone,
                            <runUpdateWeights_t>RunUpdateWeights,
                            <runUpdatePrimalSolution_t>RunUpdatePrimalSolution)
        return ret

    cdef double updateWeights(self, CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn):
        cyinp = CyCoinIndexedVector()
        cyinp.setCppSelf(inp)
        cyspare = CyCoinIndexedVector()
        cyspare.setCppSelf(spare)
        cyspare2 = CyCoinIndexedVector()
        cyspare2.setCppSelf(spare2)
        cyupdatedColumn = CyCoinIndexedVector()
        cyupdatedColumn.setCppSelf(updatedColumn)
        return self.dualPivotMethodObject.updateWeights(cyinp, cyspare, cyspare2, cyupdatedColumn)

    cdef void updatePrimalSolution(self,
                                   CppCoinIndexedVector* inp,
                                   double theta,
                                   np.ndarray[np.double_t,ndim=1] changeInObjective):
        cyinp = CyCoinIndexedVector()
        cyinp.setCppSelf(inp)
        change = self.dualPivotMethodObject.updatePrimalSolution(cyinp, theta, changeInObjective)
        changeInObjective[0] = change
